#!/usr/bin/env python
# -*- no-plot -*-
"""
Multiline text example
"""

import gr
from math import sin, cos, pi
import time

hor_align = {'Left':1, 'Center':2, 'Right':3}
vert_align = {'Top':1, 'Cap':2, 'Half':3, 'Base':4, 'Bottom':5}

gr.selntran(0)
gr.setcharheight(0.024)

for angle in range(361):

  gr.setcharup(sin(-angle * pi/180), cos(-angle * pi/180))
  gr.setmarkertype(2)
  gr.clearws()

  for halign in hor_align:
    for valign in vert_align:
      gr.settextalign(hor_align[halign], vert_align[valign])
      x = -0.1 + hor_align[halign] * 0.3;
      y = 1.1 - vert_align[valign] * 0.2;
      s = halign + '\n' + valign + '\n' + 'third line'
      gr.polymarker([x], [y])
      gr.text(x, y, s)
      tbx, tby = gr.inqtext(x, y, s)
      gr.fillarea(tbx, tby)

  gr.updatews()
  time.sleep(0.02)



#!/usr/bin/env python
# -*- animation -*-
"""
Animation of a double pendulum
"""

from numpy import sin, cos, pi, array
import time
import gr

g = 9.8        # gravitational constant

def rk4(x, h, y, f):
    k1 = h * f(x, y)
    k2 = h * f(x + 0.5 * h, y + 0.5 * k1)
    k3 = h * f(x + 0.5 * h, y + 0.5 * k2)
    k4 = h * f(x + h, y + k3)
    return x + h, y + (k1 + 2 * (k2 + k3) + k4) / 6.0

def pendulum_derivs(t, state):
    # The following derivation is from:
    # http://scienceworld.wolfram.com/physics/DoublePendulum.html
    t1, w1, t2, w2 = state
    a = (m1 + m2) * l1
    b = m2 * l2 * cos(t1 - t2)
    c = m2 * l1 * cos(t1 - t2)
    d = m2 * l2
    e = -m2 * l2 * w2**2 * sin(t1 - t2) - g * (m1 + m2) * sin(t1)
    f =  m2 * l1 * w1**2 * sin(t1 - t2) - m2 * g * sin(t2)
    return array([w1, (e*d-b*f) / (a*d-c*b), w2, (a*f-c*e) / (a*d-c*b)])

def pendulum(theta, length, mass):
    l = length[0] + length[1]
    gr.clearws()
    gr.setviewport(0, 1, 0, 1)
    gr.setwindow(-l, l, -l, l)
    gr.setmarkertype(gr.MARKERTYPE_SOLID_CIRCLE)
    gr.setmarkercolorind(86)
    pivot = [0, 0.775]                         # draw pivot point
    gr.fillarea([-0.2, 0.2, 0.2, -0.2], [0.75, 0.75, 0.8, 0.8])
    for i in range(2):
        x = [pivot[0], pivot[0] + sin(theta[i]) * length[i]]
        y = [pivot[1], pivot[1] - cos(theta[i]) * length[i]]
        gr.polyline(x, y)                   # draw rod
        gr.setmarkersize(3 * mass[i])
        gr.polymarker([x[1]], [y[1]])       # draw bob
        pivot = [x[1], y[1]]
    gr.updatews()
    return

l1 = 1.2       # length of rods
l2 = 1.0
m1 = 1.0       # weights of bobs
m2 = 1.5
t1 = 100.0     # inintial angles
t2 = -20.0

w1 = 0.0
w2 = 0.0
t = 0
dt = 0.04
state = array([t1, w1, t2, w2]) * pi / 180

now = time.clock()

while t < 30:
    start = now

    t, state = rk4(t, dt, state, pendulum_derivs)
    t1, w1, t2, w2 = state
    pendulum([t1, t2], [l1, l2], [m1, m2])

    now = time.clock()
    if start + dt > now:
        time.sleep(start + dt - now)




MAX_TNR = 9
MWIDTH = 0.254
MHEIGHT = 0.1905
WIDTH = 1024
HEIGHT = 768
GKS_K_INTSTYLE_HOLLOW = 0
GKS_K_INTSTYLE_SOLID = 1
GKS_K_INTSTYLE_PATTERN = 2
GKS_K_INTSTYLE_HATCH = 3
PATTERNS = 120
HATCH_STYLE = 108
GKS_K_TEXT_PRECISION_STRING = 0
MAX_COLOR = 1256
GKS_K_TEXT_VALIGN_NORMAL = 0
GKS_K_TEXT_VALIGN_TOP = 1
GKS_K_TEXT_VALIGN_CAP = 2
GKS_K_TEXT_VALIGN_HALF = 3
GKS_K_TEXT_VALIGN_BASE = 4
GKS_K_TEXT_VALIGN_BOTTOM = 5
GKS_K_TEXT_HALIGN_NORMAL = 0
GKS_K_TEXT_HALIGN_LEFT = 1
GKS_K_TEXT_HALIGN_CENTER = 2
GKS_K_TEXT_HALIGN_RIGHT = 3
GKS_K_NOCLIP = 0
GKS_K_CLIP = 1



#!/usr/bin/env python
# -*- animation -*-
"""
Sample microphone and display input signal in realtime
"""

import pyaudio
import numpy
import gr
import time
from gr.pygr import *

FS=44100       # Sampling frequency
SAMPLES=1000

mic = None
def get_audio_data():
    global mic
    if mic is None:
        pa = pyaudio.PyAudio()
        mic = pa.open(format=pyaudio.paInt16, channels=1, rate=FS,
                      input=True, frames_per_buffer=SAMPLES)
    amplitudes = numpy.fromstring(mic.read(SAMPLES), dtype=numpy.short)
    return amplitudes / 32768.

start = time.time()

while time.time() - start < 10:
    try:
        amplitudes = get_audio_data()
    except (IOError):
        continue
    plot(range(SAMPLES), amplitudes, window=[0, SAMPLES, -1, 1])




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
GR / wxPython interoperability example
"""

import wx
import os
import gr

# get_address = lambda obj_str: obj_str[obj_str.find('0x'): obj_str.find('>', obj_str.find('0x'))]

def get_address(obj_str):
    digits = '0123456789abcdef'
    obj_str = obj_str.lower()
    pos = obj_str.find('0x') + 2
    if pos >= 0:
        tmp_string = '0x'
        while pos < len(obj_str) and obj_str[pos] in digits:
            tmp_string += obj_str[pos]
            pos += 1
        return int(tmp_string, 16)
    else:
        return None

class GrWidget(wx.Panel):
    def __init__(self, parent, id):
        wx.Panel.__init__(self, parent, id)
        
        self.bm = None
        
        self.setupUi()

        os.environ['GKS_WSTYPE'] = "380"
        os.environ['GKS_DOUBLE_BUF'] = "True"

        self.DrawButton.Bind(wx.EVT_BUTTON, self.draw)
        self.QuitButton.Bind(wx.EVT_BUTTON, self.quit)
        self.Bind(wx.EVT_PAINT, self.paintEvent)
                
    def setupUi(self):
        self.DrawButton = wx.Button(self, 1, 'Draw')
        self.DrawButton.SetPosition((300, 10))
        self.QuitButton = wx.Button(self, 2, 'Quit')
        self.QuitButton.SetPosition((400, 10))

    def quit(self, event):
        gr.emergencyclosegks()
        self.GetParent().Destroy()

    def draw(self, event):
        self.bm = wx.EmptyBitmap(self.GetSizeTuple()[0], self.GetSizeTuple()[1])
        dc = wx.MemoryDC(self.bm)
        
        os.environ['GKSconid'] = "%x!%x" % (get_address(repr(self)), get_address(repr(dc)))
        
        dc.SetBackground(wx.WHITE_BRUSH)
        dc.Clear()
        
        dc.DrawText("Surface Plot using wxWidgets ...", 15, 15)
        
        x = range(1, 481)
        y = range(1, 481)
        w, h, d = gr.readimage(
            os.path.join(os.path.dirname(os.path.realpath(__file__)),
                         'surf.png'))
        z = map(lambda x: x & 0xff, d)

        gr.setviewport(0, 1, 0, 1)
        gr.setwindow(1, 480, 1, 480)
        gr.setspace(1, 1000, 30, 80)
        gr.setcolormap(3)
        gr.surface(x, y, z, 6)
        gr.contour(x, y, range(1), z, 0)
        gr.updatews()
        
        self.Refresh()
        event.Skip()

    def paintEvent(self, ev):
        painter = wx.PaintDC(self)
        
        if self.bm != None:
            painter.DrawBitmap(self.bm, 0, 0)
            
class MainFrame(wx.Frame):
    def __init__(self, parent, id, title):
        wx.Frame.__init__(self, parent, id, title=title)
        self.widget = GrWidget(self, -1)
        self.widget.SetSize((500, 500))
        self.Fit()
        self.Centre()

if __name__ == '__main__':
    app = wx.App()

    win = MainFrame(None, -1, title='GRWidget')
    win.Show()

app.MainLoop()



#!/usr/bin/env python
# -*- animation -*-
"""
Solving The 2D Diffusion Equation
"""

import numpy
import gr, time
from numba.decorators import jit

dx = 0.005
dy = 0.005
a = 0.5
dt = dx*dx*dy*dy/(2*a*(dx*dx+dy*dy))
timesteps = 150

nx = int(1/dx)
ny = int(1/dy)
ui = numpy.zeros([nx,ny])
u = numpy.zeros([nx,ny])

for i in range(nx):
    for j in range(ny):
        if (((i*dx-0.5)**2+(j*dy-0.5)**2 <= 0.1) &
            ((i*dx-0.5)**2+(j*dy-0.5)**2 >= 0.05) ):
            ui[i,j] = 1

def diff_step(u, ui):
    for i in range(1,nx-1):
        for j in range(1,ny-1):
            uxx = ( ui[i+1,j] - 2*ui[i,j] + ui[i-1, j] )/(dx*dx)
            uyy = ( ui[i,j+1] - 2*ui[i,j] + ui[i, j-1] )/(dy*dy)
            u[i,j] = ui[i,j]+dt*a*(uxx+uyy)

diff_step_numba = jit('void(f8[:,:], f8[:,:])')(diff_step)

now = time.clock()

t = 0
worker = 'CPython'

for m in range(timesteps):
    gr.clearws()

    start = now
    if t > 5:
        diff_step_numba(u, ui)
        worker = 'Numba'
    else:
        diff_step(u, ui)
    ui = numpy.copy(u)
    now = time.clock()
    t = t + now - start

    c = 1000 + 255 * u.ravel()
    gr.setviewport(0, 1, 0, 1)
    gr.setcolormap(-32)
    gr.cellarray(0, 1, 0, 1, nx, ny, c)
    gr.text(0.01, 0.95, '%10s: %7.2f fps' % (worker, 1.0 / (now - start)))
    gr.updatews()



# -*- coding: utf-8 -*-
"""..."""
# standard library
import logging
# third party
def _importPySide():
    global QtCore
    from PySide import QtCore

def _importPyQt4():
    global QtCore
    from PyQt4 import QtCore

from qtgr.backend import QT_BACKEND_ORDER, QT_PYSIDE, QT_PYQT4
_imp = {QT_PYSIDE: _importPySide,
        QT_PYQT4: _importPyQt4}
try:
    _imp[QT_BACKEND_ORDER[0]]()
except ImportError:
    _imp[QT_BACKEND_ORDER[1]]()
# local library
import gr
import qtgr
from gr.pygr import DeviceCoordConverter
from gr._version import __version__, __revision__

__author__ = "Christian Felder <c.felder@fz-juelich.de>"
__copyright__ = """Copyright 2012-2014 Forschungszentrum Juelich GmbH

This file is part of GR, a universal framework for visualization applications.
Visit http://gr-framework.org for the latest
version.

GR was developed by the Scientific IT-Systems group at the Peter Grünberg
Institute at Forschunsgzentrum Jülich. The main development has been done
by Josef Heinen who currently maintains the software.

GR is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GR. If not, see <http://www.gnu.org/licenses/>.
 
"""

_log = logging.getLogger(__name__)

class EventMeta(QtCore.QEvent):

    def __init__(self, type):
        if isinstance(type, int):
            type = QtCore.QEvent.Type(type)
        super(EventMeta, self).__init__(type)
        self._type = type

    def type(self):
        return self._type

class MouseLocationEventMeta(EventMeta):

    def __init__(self, type, width, height, x, y, window=None):
        super(MouseLocationEventMeta, self).__init__(type)
        self._coords = DeviceCoordConverter(width, height, window=window)
        self._coords.setDC(x, y)

    def getWindow(self):
        return self._coords.getWindow()

    def getWC(self, viewport):
        return self._coords.getWC(viewport)

    def getNDC(self):
        return self._coords.getNDC()

    def getDC(self):
        return self._coords.getDC()



#!/usr/bin/env python
# -*- no-plot -*-
"""
Calculate Mandelbrot set using NumbaPro (GPU version)
"""

from numbapro import *
from timeit import default_timer as timer

import numpy as np
import gr

@cuda.jit(restype=uint32, argtypes=[f8, f8, uint32], device=True)
def mandel(x, y, max_iters):
    c = complex(x, y)
    z = 0.0j
    ci = 0
    inc = 1

    for i in range(max_iters):
        z = z * z + c
        if (z.real * z.real + z.imag * z.imag) >= 4:
            return ci
        ci += inc
        if ci == 0 or ci == 255:
            inc = -inc

    return 255

@cuda.jit(argtypes=[f8, f8, f8, f8, uint8[:,:], uint32])
def mandel_kernel(min_x, max_x, min_y, max_y, image, max_iters):
    height = image.shape[0]
    width = image.shape[1]

    pixel_size_x = (max_x - min_x) / width
    pixel_size_y = (max_y - min_y) / height

    startX = cuda.blockDim.x * cuda.blockIdx.x + cuda.threadIdx.x
    startY = cuda.blockDim.y * cuda.blockIdx.y + cuda.threadIdx.y
    gridX = cuda.gridDim.x * cuda.blockDim.x;
    gridY = cuda.gridDim.y * cuda.blockDim.y;

    for x in range(startX, width, gridX):
        real = min_x + x * pixel_size_x
        for y in range(startY, height, gridY):
            imag = min_y + y * pixel_size_y 
            image[y, x] = mandel(real, imag, max_iters)


x = -0.9223327810370947027656057193752719757635
y = 0.3102598350874576432708737495917724836010

image = np.zeros((500, 500), dtype = np.uint8)
blockdim = (32, 8)
griddim = (32, 16)

f = 0.5
for i in range(200):
    start = timer()
    d_image = cuda.to_device(image)
    mandel_kernel[griddim, blockdim](x-f, x+f, y-f, y+f, d_image, 400) 
    d_image.to_host()
    dt = timer() - start

    print("Mandelbrot created in %f s" % dt)
    ca = 1000.0 + image.ravel()

    gr.clearws()
    gr.setviewport(0, 1, 0, 1)
    gr.setcolormap(13)
    gr.cellarray(0, 1, 0, 1, 500, 500, ca)
    gr.updatews()

    f *= 0.9



import sys
import zmq
import parser
from struct import unpack

context = zmq.Context()
socket = context.socket(zmq.PULL)
socket.connect("tcp://localhost:5556")

while True:
    len_data = socket.recv()
    length = unpack('i', len_data)[0]
    data = ''
    while len(data) < length:
        chunk = socket.recv()
        if chunk == '':
            break
        data += chunk
    if len(data) != length:
        break
    parser.interp(data)



import os
import sys
import shutil

THIS_DIR = os.path.dirname(os.path.abspath(__file__))
DOC_DIR = os.path.abspath(os.path.join(THIS_DIR, '..'))
EXAMPLES_DIR = os.path.abspath(os.path.join(DOC_DIR, '..', 'examples'))
OUTPUT_DIR = os.path.join(DOC_DIR, 'examples')
MEDIA_DIR = os.path.abspath(os.path.join(DOC_DIR, 'media'))


def clean():
    if os.path.isdir(OUTPUT_DIR):
        shutil.rmtree(OUTPUT_DIR)


def get_example_filenames(examples_dir):

    for (dirpath, dirnames, filenames) in os.walk(examples_dir):
        for fname in filenames:
            if not fname.endswith('.py'):
                continue
            filename = os.path.join(dirpath, fname)
            name = filename[len(examples_dir):].lstrip('/\\')[:-3]
            name = name.replace('\\', '/')
            f = open(filename, 'r')
            line = f.readline()
            f.close()
            if line.startswith('#!/usr/bin/env python'):
                yield filename, name


def movie(name):
    lines = []
    lines.append('.. raw:: html')
    lines.append('')
    lines.append('   <script language="javascript">')
    lines.append('   QT_WriteOBJECT("/media/%s.mov" , "558", "558" , "", "autoplay", "true", "controller", "false");' % name)
    lines.append('   </script>')
    return lines

def image(name):
    lines = []
    lines.append('.. image:: %s.png' % name)
    return lines

def separator():
    lines = []
    lines.append('')
    lines.append('----')
    return lines

def create_examples(examples):

    contents = []

    # Create doc file for each example
    for filename, name in examples:
        print('Writing example %s' % name)

        # Create title
        lines = []
        lines.append(name)
        lines.append('-' * len(lines[-1]))
        lines.append('')

        # Get source
        doclines = []
        infoline = ''
        sourcelines = []
        animation = False
        plot = True
        with open(os.path.join(EXAMPLES_DIR, name + '.py')) as f:
            for line in f.readlines():
                line = line.rstrip()
                if line.startswith('# -*- no-plot -*-') or 'PySide' in line or 'PyQt4' in line or 'wxPython' in line:
                    plot = False
                elif line.startswith('# -*- animation -*-'):
                    animation = True
                if not doclines:
                    if line.startswith('"""'):
                        doclines.append(line.lstrip('" '))
                        sourcelines = []
                    else:
                        sourcelines.append('    ' + line)
                elif not sourcelines:
                    if '"""' in line:
                        sourcelines.append('    ' + line.partition('"""')[0])
                    else:
                        if infoline == '':
                            infoline = line
                        doclines.append(line)
                else:
                    sourcelines.append('    ' + line)

        # Add desciprion
        lines.extend(doclines)
        lines.append('')

        # Add image or screencast
        if plot:
            if animation:
                lines.extend(movie(name))
            else:
                lines.extend(image(name))
            lines.extend(separator())
        else:
            path = os.path.join(MEDIA_DIR, name)
            if os.access(path + '.png', os.R_OK):
                lines.extend(image(name))
                lines.extend(separator())
                shutil.copy(path + '.png', '%s.png' % os.path.join(OUTPUT_DIR, name))
            elif os.access(path + '.mov', os.R_OK):
                lines.extend(movie(name))
                lines.extend(separator())
                shutil.copy(path + '.mov', '%s.mov' % os.path.join(OUTPUT_DIR, name))

        # Add source code
        lines.append('')
        lines.append('.. code-block:: python')
        lines.append('    ')
        lines.extend(sourcelines)
        lines.append('')

        # Write
        output_filename = os.path.join(OUTPUT_DIR, name + '.rst')
        output_dir = os.path.dirname(output_filename)
        if not os.path.isdir(output_dir):
            os.mkdir(output_dir)
        with open(output_filename, 'w') as f:
            f.write('\n'.join(lines))

        if plot:
            if animation:
                mov_file = os.path.join(MEDIA_DIR, name + '.mov')
                if not os.access(mov_file, os.R_OK):
                    print('Create MOV animation for example %s' % name)
                    os.popen('gr -t mov %s.py' % os.path.join(EXAMPLES_DIR, name))
                    shutil.move('gks.mov', mov_file)
            else:
                print('Create PNG figure for example %s' % name)
                os.popen('gr -t png %s.py' % os.path.join(EXAMPLES_DIR, name))
                shutil.move('gks_p001.png', '%s.png' % os.path.join(OUTPUT_DIR, name))
        contents.append([filename, name, infoline])

    return contents


def create_toc(contents):

    # Create TOC
    lines = []
    lines.append('Examples')
    lines.append('=' * len(lines[-1]))
    lines.append('')

    # Add entry for each example that we know
    lines.append('.. toctree::')
    lines.append('')
    for _, name, info in contents:
        lines.append('   %s <%s>' % (info, name))
    lines.append('')

    # Write file
    with open(os.path.join(DOC_DIR, 'examples', 'index.rst'), 'w') as f:
        f.write('\n'.join(lines))


def main():

    # Get examples and sort
    examples = list(get_example_filenames(EXAMPLES_DIR))
    examples.sort(key=lambda x: x[1])

    contents = create_examples(examples)
    create_toc(contents)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""..."""
# standard library
import logging
# third party
# ...
# local library
import gr
from gr.pygr.helper import ColorIndexGenerator
from gr._version import __version__, __revision__

__author__ = "Christian Felder <c.felder@fz-juelich.de>"
__copyright__ = """Copyright 2012 - 2014 Forschungszentrum Juelich GmbH

This file is part of GR, a universal framework for visualization applications.
Visit http://gr-framework.org for the latest
version.

GR was developed by the Scientific IT-Systems group at the Peter Grünberg
Institute at Forschunsgzentrum Jülich. The main development has been done
by Josef Heinen who currently maintains the software.

GR is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GR. If not, see <http://www.gnu.org/licenses/>.
 
"""

_log = logging.getLogger(__name__)

class GRMeta(object):

    def drawGR(self):
        raise NotImplementedError

class GRDrawAttributes(object):

    def __init__(self, linetype=gr.LINETYPE_SOLID, markertype=gr.MARKERTYPE_DOT,
                 linecolor=None, markercolor=1):
        self._linetype, self._markertype = linetype, markertype
        self._linecolor, self._markercolor = linecolor, markercolor
        if linecolor is None:
            self._linecolor = ColorIndexGenerator.nextColorIndex()
        else:
            self._linecolor = linecolor

    @property
    def linetype(self):
        """Get current linetype."""
        return self._linetype

    @linetype.setter
    def linetype(self, value):
        self._linetype = value

    @property
    def markertype(self):
        """Get current markertype."""
        return self._markertype

    @markertype.setter
    def markertype(self, value):
        self._markertype = value

    @property
    def linecolor(self):
        """Get current linecolor."""
        return self._linecolor

    @linecolor.setter
    def linecolor(self, value):
        self._linecolor = value

    @property
    def markercolor(self):
        """Get current markercolor."""
        return self._markercolor

    @markercolor.setter
    def markercolor(self, value):
        self._markercolor = value

class GRViewPort(object):

    DEFAULT_VIEWPORT = (.1, .95, .1, .95)

    def __init__(self, viewport=DEFAULT_VIEWPORT):
        self._viewport = list(viewport)
        self._sizex, self._sizey = 1., 1.

    @property
    def sizex(self):
        """..."""
        return self._sizex

    @sizex.setter
    def sizex(self, value):
        self._sizex = value

    @property
    def sizey(self):
        """..."""
        return self._sizey

    @sizey.setter
    def sizey(self, value):
        self._sizey = value

    @property
    def viewport(self):
        """get current viewport"""
        return self._viewport

    @viewport.setter
    def viewport(self, viewport):
        self._viewport = list(viewport)

    @property
    def viewportscaled(self):
        vp = list(self.viewport)
        vp[0] *= self.sizex
        vp[1] *= self.sizex
        vp[2] *= self.sizey
        vp[3] *= self.sizey
        return vp




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import with_statement, print_function

import sys
import os
import shlex
import re
import tempfile
import distutils.command
import distutils.command.install
from distutils.core import Command
from distutils.command.build_ext import build_ext as _build_ext
from distutils.command.build import build as _build
from distutils.command.clean import clean as _clean
from distutils.core import setup, Extension
from distutils.ccompiler import new_compiler
from distutils.sysconfig import get_config_var
from distutils.sysconfig import get_python_lib
from distutils.util import get_platform
from subprocess import PIPE, STDOUT
if str == bytes:
    from subprocess import Popen
else:
    # since python 3.x str and bytes type are not equal.
    # the bytes type has no read attribute which is needed,
    # e.g. for rstrip calls
    from subprocess import Popen as _Popen


    class Popen(_Popen):

        def communicate(self, input=None, timeout=None):
            stdout, stderr = _Popen.communicate(self, input, timeout)
            if stdout is not None:
                stdout = stdout.decode()
            if stderr is not None:
                stderr = stderr.decode()
            return (stdout, stderr)
import vcversioner


__author__ = "Christian Felder <c.felder@fz-juelich.de>"
__version__ = vcversioner.find_version(
                   version_module_paths=[os.path.join("lib", "gr", "python",
                                                      "gr",
                                                      "_version.py")]).version
__copyright__ = """Copyright 2012 - 2014 Forschungszentrum Juelich GmbH

This file is part of GR, a universal framework for visualization applications.
Visit http://gr-framework.org for the latest
version.

GR was developed by the Scientific IT-Systems group at the Peter Grünberg
Institute at Forschunsgzentrum Jülich. The main development has been done
by Josef Heinen who currently maintains the software.

GR is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GR. If not, see <http://www.gnu.org/licenses/>.

"""


class build_static(Command):

    # prerequisites: build static 3rdparty libraries
    description = "build static 3rdparty libraries"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if sys.platform == "win32":
            _extra_preargs = list(_msvc_extra_compile_args)
        else:
            _extra_preargs = ["-fPIC"]
        if not os.path.isdir("build"):
            os.mkdir("build")
        if not os.path.isdir(_build_3rdparty):
            os.mkdir(_build_3rdparty)
        if not os.path.isdir(_build_temp):
            os.mkdir(_build_temp)
        compiler = new_compiler()
        if not os.path.isfile(_libz):
            obj = compiler.compile(_libz_src_path, extra_preargs=_extra_preargs)
            compiler.create_static_lib(obj, "z", output_dir=_build_3rdparty)
        if not os.path.isfile(_libpng):
            _png_extra_preargs = list(_extra_preargs)
            _png_extra_preargs.append("-I3rdparty/zlib")
            obj = compiler.compile(_libpng_src_path,
                                   extra_preargs=_png_extra_preargs)
            compiler.create_static_lib(obj, "png", output_dir=_build_3rdparty)
        if not os.path.isfile(_libjpeg):
            obj = compiler.compile(_libjpeg_src_path,
                                   extra_preargs=_extra_preargs)
            compiler.create_static_lib(obj, "jpeg", output_dir=_build_3rdparty)
#        # create batch scripts
#        if sys.platform == "win32":
#            _grvarsallbat = os.path.join(_build_temp, "grvarsall.bat")
#            with open(_grvarsallbat, "wb") as fd:
#                paths = ["%PATH%"]
#                fd.write("@echo off\r\n")
#                if _wxlib:
#                    fd.write("set WXLIB=%s\r\n" %_wxlib)
#                    paths.append("%WXLIB%")
#                if _gsdir:
#                    fd.write("set GSDIR=%s\r\n" %_gsdir)
#                    paths.append("%GSDIR%\\bin")
#                fd.write("set PATH=%s\r\n" %(';'.join(paths)))


class clean_static(Command):

    description = "clean up temporary files from 'build_static' command"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print("removing '", _build_3rdparty, "' (and everything under it)")
        try:
            map(lambda p: os.remove(os.path.join(_build_3rdparty, p)),
                os.listdir(_build_3rdparty))
            os.rmdir(_build_3rdparty)
        except OSError:
            pass


class clean(_clean, clean_static):

    def run(self):
        clean_static.run(self)
        _clean.run(self)
        if sys.platform == "darwin":
            os.system("xcodebuild -project lib/gks/quartz/GKSTerm.xcodeproj clean")


class check_ext(Command):

    description = "dynamically generate list of C/C++ extensions for build_ext"

    # List of option tuples: long name, short name (None if no short
    # name), and help string.
    user_options = [('disable-wx', None, "Disable wx plugin"),
                    ('disable-qt', None, "Disable qt plugin"),
                    ('disable-gtk', None, "Disable gtk plugin"),
                    ('disable-gs', None, "Disable gs plugin"),
                    ('disable-fig', None, "Disable fig plugin"),
                    ('disable-svg', None, "Disable svg plugin"),
                    ('disable-wmf', None, "Disable wmf plugin"),
                    ('disable-mov', None, "Disable mov plugin"),
                    ('disable-html', None, "Disable html plugin"),
                    ('disable-opengl', None, "Disable OpenGL libraries"),
                    ('disable-quartz', None,
                     "Disable quartz plugin (OSX only)"),
                    ('disable-x11', None, "Disable X11 libraries"),
                    ('disable-freetype', None,
                     "Disable freetype libraries (disables also mupdf)"),
                    ('disable-xft', None,
                     "Disable Xft libraries (disables also freetype)"),
                    ('disable-xt', None,
                     "Disable Xt libraries (disables also x11 and freetype)"),
                    ('disable-mupdf', None, "Disable mupdf libraries"),
                   ]

    def initialize_options(self):
        self.isLinuxOrDarwin = any(s in sys.platform for s in
                                   ["linux", "darwin"])
        self.isLinux = ("linux" in sys.platform)
        self.isDarwin = (sys.platform == "darwin")
        self.isWin32 = (sys.platform == "win32")
        # extensions list
        self.ext_modules = []
        # -- user options -------------------------------------
        self.disable_wx = False
        self.disable_qt = False
        self.disable_gs = False
        self.disable_fig = False
        self.disable_svg = False
        self.disable_html = False
        self.disable_wmf = False
        self.disable_opengl = False
        # only available on OS X
        self.disable_quartz = False if self.isDarwin else True
        # available on Linux and OS X
        self.disable_gtk = False if self.isLinuxOrDarwin else True
        self.disable_x11 = False if self.isLinuxOrDarwin else True
        self.disable_xft = False if self.isLinuxOrDarwin else True
        self.disable_xt = False if self.isLinuxOrDarwin else True
        self.disable_freetype = False if self.isLinuxOrDarwin else True
        self.disable_mupdf = False if self.isLinuxOrDarwin else True
        self.disable_mov = False if self.isLinuxOrDarwin else True
        # -- environment -------------------------------------
        self.cc = os.getenv("CC", "cc")
        self.grdir = os.getenv("GRDIR",
                               os.path.join(get_python_lib(plat_specific=True,
                                                           prefix=''), "gr"))
        # -- x11 -------------------------------------
        self.x11lib = []
        self.x11inc = []
        self.x11libs = []
        self.x11ldflags = []
        self.x11cflags = []
        # -- gs -------------------------------------
        self.gsdir = os.getenv("GSDIR")
        self.gsinc = []
        self.gslib = []
        self.gslibs = []
        self.gsldflags = []
        # -- wx -------------------------------------
        self.wxconfig = os.getenv("WX_CONFIG")
        self.wxdir = os.getenv("WXDIR")
        self.wxlib = [os.getenv("WXLIB")] if os.getenv("WXLIB") else []
        self.wxinc = []
        self.wxlibs = []
        self.wxldflags = []
        self.wxcxx = []
        # -- qt -------------------------------------
        if not self.isWin32:
            self.qtdir = os.getenv("QTDIR", "/usr/local")
        else:
            self.qtdir = os.getenv("QTDIR", "")
        self.qtdir = os.getenv("QTDIR", "/usr/local")
        self.qtinc = [os.path.join(self.qtdir, "include")]
        self.qtlib = [os.path.join(self.qtdir, "lib")]
        self.qtldflags = []
        self.qmake = ""
        self.qtversion = None
        qmake = os.path.join(self.qtdir, "bin", "qmake")
        if self.isWin32:
            qmake += ".exe"
        if os.path.isfile(qmake):
            self.qmake = qmake
        # -- freetype -------------------------------------
        self.ftconfig = None
        self.ftldflags = []
        self.ftcflags = []
        # -- gtk -------------------------------------
        self.gtk_package = "gtk+-2.0"
        self.gtkldflags = []
        self.gtkcflags = []
        # -- mupdf -------------------------------------
        self.mupdfinc = []
        self.mupdflibs = []
        # -- opengl -------------------------------------
        self.gllibs = []
        self.glldflags = []
        # -- platform extra link args -------------------------------------
        self.platform_ldflags = []

    def _test_gs(self, gsinc=[], gslibs=[], gscflags=[], gsldflags=[]):
        cflags = ["-I" + i for i in gsinc]
        ldflags = ["-l" + l for l in gslibs]
        (fd, tmpsrc) = tempfile.mkstemp(suffix=".c", prefix="a")
        (_fd2, tmpout) = tempfile.mkstemp(suffix=".out", prefix="a")
        os.write(fd, b"""#include <stdio.h>
#include <stdlib.h>
#include <ghostscript/iapi.h>

int main()
{
  gsapi_revision_t r;
  if (gsapi_revision(&r, sizeof(gsapi_revision_t)) == 0)
    fprintf(stderr, \"%ld\\n\", r.revision);
  exit(0);
}

""")
        os.close(fd)
        cmd = [self.cc, "-o", tmpout, tmpsrc]
        cmd.extend(cflags)
        cmd.extend(gscflags)
        cmd.extend(ldflags)
        cmd.extend(gsldflags)
        gscc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        gscc.communicate()
        try:
            os.remove(tmpsrc)
            os.remove(tmpout)
        except OSError:
            pass
        return gscc.returncode == 0

    def _test_mupdf(self, mupdfinc=[], mupdflibs=[], mupdfcflags=[],
                    mupdfldflags=[]):
        cflags = ["-I" + i for i in mupdfinc]
        ldflags = ["-l" + l for l in mupdflibs]
        (fd, tmpsrc) = tempfile.mkstemp(suffix=".c", prefix="a")
        (_fd2, tmpout) = tempfile.mkstemp(suffix=".out", prefix="a")
        os.write(fd, b"""#include <mupdf/fitz.h>

int main(int argc, char **argv)
{
  fz_context  *ctx;
  fz_document *doc;
  fz_rect      rect;
  fz_irect     bbox;
  fz_pixmap   *pix;
  fz_device   *dev;
  fz_page     *page;

  ctx = fz_new_context(NULL, NULL, FZ_STORE_UNLIMITED);
  fz_register_document_handlers(ctx);
  doc = fz_open_document(ctx, argv[1]);
  page = fz_load_page(doc, 0);
  fz_bound_page(doc, page, &rect);
  fz_round_rect(&bbox, &rect);
  pix = fz_new_pixmap_with_bbox(ctx, fz_device_rgb(ctx), &bbox);
  dev = fz_new_draw_device(ctx, pix);
  fz_run_page(doc, page, dev, &fz_identity, NULL);

  fz_free_device(dev);
  fz_drop_pixmap(ctx, pix);
  fz_free_page(doc, page);
  fz_free_context(ctx);
  return 0;
}

""")
        os.close(fd)
        cmd = [self.cc, "-o", tmpout, tmpsrc]
        cmd.extend(cflags)
        cmd.extend(mupdfcflags)
        cmd.extend(ldflags)
        cmd.extend(mupdfldflags)
        gscc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        gscc.communicate()
        try:
            os.remove(tmpsrc)
            os.remove(tmpout)
        except OSError:
            pass
        return gscc.returncode == 0

    def _test_mov(self, mupdfinc=[], mupdflibs=[], mupdfcflags=[],
                  mupdfldflags=[]):
        cflags = ["-I" + i for i in mupdfinc]
        ldflags = ["-l" + l for l in mupdflibs]
        (fd, tmpsrc) = tempfile.mkstemp(suffix=".c", prefix="a")
        (_fd2, tmpout) = tempfile.mkstemp(suffix=".out", prefix="a")
        os.write(fd, b"""#include <stdio.h>
#include <stdlib.h>
#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libavutil/mathematics.h>
#include <libavutil/imgutils.h>
#include <libswscale/swscale.h>
#include <mupdf/fitz.h>

int main()
{
  exit(0);
}

""")
        os.close(fd)
        cmd = [self.cc, "-o", tmpout, tmpsrc]
        cmd.extend(cflags)
        cmd.extend(mupdfcflags)
        cmd.extend(ldflags)
        cmd.extend(mupdfldflags)
        gscc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        gscc.communicate()
        try:
            os.remove(tmpsrc)
            os.remove(tmpout)
        except OSError:
            pass
        return gscc.returncode == 0

    def _test_opengl(self, glinc=[], gllibs=[], glcflags=[], glldflags=[]):
        cflags = ["-I" + i for i in glinc]
        ldflags = ["-l" + l for l in gllibs]
        (fd, tmpsrc) = tempfile.mkstemp(suffix=".c", prefix="a")
        (_fd2, tmpout) = tempfile.mkstemp(suffix=".out", prefix="a")
        os.write(fd, b"""#include <stdio.h>
#include <stdlib.h>
#ifdef __APPLE__
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif

int main()
{
  exit(0);
}

""")
        os.close(fd)
        cmd = [self.cc, "-o", tmpout, tmpsrc]
        cmd.extend(cflags)
        cmd.extend(glcflags)
        cmd.extend(ldflags)
        cmd.extend(glldflags)
        gscc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        gscc.communicate()
        try:
            os.remove(tmpsrc)
            os.remove(tmpout)
        except OSError:
            pass
        return gscc.returncode == 0

    def _test_xft(self, xftinc=[], xftlib=[], xftlibs=[], xftcflags=[],
                  xftldflags=[]):
        cflags = ["-I" + i for i in xftinc]
        ldflags = ["-L" + ld for ld in xftlib]
        ld = ["-l" + l for l in xftlibs]
        ldflags.extend(ld)
        (fd, tmpsrc) = tempfile.mkstemp(suffix=".c", prefix="a")
        (_fd2, tmpout) = tempfile.mkstemp(suffix=".out", prefix="a")
        os.write(fd, b"""#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>

#include <X11/Xft/Xft.h>

int main()
{
  exit(0);
}

""")
        os.close(fd)
        cmd = [self.cc, "-o", tmpout, tmpsrc]
        cmd.extend(cflags)
        cmd.extend(xftcflags)
        cmd.extend(ldflags)
        cmd.extend(xftldflags)
        gscc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        gscc.communicate()
        try:
            os.remove(tmpsrc)
            os.remove(tmpout)
        except OSError:
            pass
        return gscc.returncode == 0

    def _test_xt(self, xtinc=[], xtlib=[], xtlibs=[], xtcflags=[],
                 xtldflags=[]):
        cflags = ["-I" + i for i in xtinc]
        ldflags = ["-L" + ld for ld in xtlib]
        ld = ["-l" + l for l in xtlibs]
        ldflags.extend(ld)
        (fd, tmpsrc) = tempfile.mkstemp(suffix=".c", prefix="a")
        (_fd2, tmpout) = tempfile.mkstemp(suffix=".out", prefix="a")
        os.write(fd, b"""#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>

#include <X11/Intrinsic.h>

int main()
{
  exit(0);
}

""")
        os.close(fd)
        cmd = [self.cc, "-o", tmpout, tmpsrc]
        cmd.extend(cflags)
        cmd.extend(xtcflags)
        cmd.extend(ldflags)
        cmd.extend(xtldflags)
        gscc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        gscc.communicate()
        try:
            os.remove(tmpsrc)
            os.remove(tmpout)
        except OSError:
            pass
        return gscc.returncode == 0

    def _test_freetype(self, ftinc=[], ftlibs=[], ftcflags=[], ftldflags=[]):
        cflags = ["-I" + i for i in ftinc]
        ldflags = ["-l" + l for l in ftlibs]
        (fd, tmpsrc) = tempfile.mkstemp(suffix=".c", prefix="a")
        (_fd2, tmpout) = tempfile.mkstemp(suffix=".out", prefix="a")
        os.write(fd, b"""#include <stdio.h>
#include <stdlib.h>

#include <ft2build.h>

int main()
{
  exit(0);
}

""")
        os.close(fd)
        cmd = [self.cc, "-o", tmpout, tmpsrc]
        cmd.extend(cflags)
        cmd.extend(ftcflags)
        cmd.extend(ldflags)
        cmd.extend(ftldflags)
        gscc = Popen(cmd, stdout=PIPE, stderr=STDOUT)
        gscc.communicate()
        try:
            os.remove(tmpsrc)
            os.remove(tmpout)
        except OSError:
            pass
        return gscc.returncode == 0

    def _finalize_LinuxOrDarwinPost(self):
        pass

    def _finalize_Win32Post(self):
        # -- qt -------------------------------------
        if not self.disable_qt:
            self.qtlibs = ["QtGui%d" % self.qtversion[0],
                           "QtCore%d" % self.qtversion[0]]

    def _finalize_LinuxOrDarwinPre(self):
        # -- wx -------------------------------------
        if not self.disable_wx and not self.wxconfig:
            self.wxconfig = Popen(["which", "wx-config"],
                              stdout=PIPE).communicate()[0].rstrip()
            if not os.path.isfile(self.wxconfig):
                self.disable_wx = True
            else:
                self.wxldflags = shlex.split(Popen([self.wxconfig, "--libs"],
                                stdout=PIPE).communicate()[0].rstrip())
                self.wxcxx = shlex.split(Popen([self.wxconfig, "--cxxflags"],
                               stdout=PIPE).communicate()[0].rstrip())
        # -- qt -------------------------------------
        if not self.disable_qt:
            if not self.qmake:
                self.qmake = Popen(["which", "qmake"],
                               stdout=PIPE).communicate()[0].rstrip()
            if os.path.isdir("/usr/local/Cellar"):
                self.qtlibs = []
            else:
                self.qtlibs = ["QtGui", "QtCore"]
        # -- x11 -------------------------------------
        if self.isLinux:
            x11ldflags = None
            try:
                x11ldflags = shlex.split(Popen(["pkg-config", "x11", "--libs"],
                   stdout=PIPE, stderr=PIPE).communicate()[0].rstrip())
            except OSError:
                pass
            if x11ldflags:
                self.x11ldflags = x11ldflags
                self.x11cflags = shlex.split(Popen(["pkg-config", "x11",
                                                    "--cflags"], stdout=PIPE,
                               stderr=PIPE).communicate()[0].rstrip())
            else:
                self.disable_x11 = True
        else:
            for p in [os.path.join(os.sep, "usr", "X11R6"),
                      os.path.join(os.sep, "usr", "X11")]:
                if os.path.isdir(p):
                    self.x11inc = [os.path.join(p, "include")]
                    self.x11lib = [os.path.join(p, "lib")]
                    break
            if not self.x11lib:
                self.disable_x11 = True
        # -- gs -------------------------------------
        if not self.disable_gs:
            for p in [os.path.join(os.sep, "usr", "local", "include"),
                      os.path.join(os.sep, "usr", "include")]:
                ghost = os.path.join(p, "ghostscript")
                if os.path.isdir(ghost):
                    self.gsinc = [ghost]
                    break
            if not self.gsinc:
                self.disable_gs = True
            else:
                self.gslibs = ["gs"] if self.isLinux else ["gs", "Xt", "X11",
                                                           "iconv"]
                if self.isDarwin:
                    self.gsldflags = ["-L" + ldir for ldir in self.x11lib]
                self.disable_gs = not self._test_gs(self.gsinc, self.gslibs,
                                                    gsldflags=self.gsldflags)
        # -- freetype -------------------------------------
        if not self.disable_freetype:
            ftconfig = Popen(["which", "freetype-config"],
                             stdout=PIPE).communicate()[0].rstrip()
            self.disable_freetype = not bool(ftconfig)
            if not self.disable_freetype:
                self.ftconfig = ftconfig
                self.ftldflags = shlex.split(Popen([self.ftconfig, "--libs"],
                               stdout=PIPE).communicate()[0].rstrip())
                self.ftcflags = shlex.split(Popen([self.ftconfig, "--cflags"],
                              stdout=PIPE).communicate()[0].rstrip())
            self.disable_freetype = (
                             not self._test_freetype(ftcflags=self.ftcflags,
                                                     ftldflags=self.ftldflags))
        # -- Xft -------------------------------------
        if not self.disable_xft:
            libs = list(self.x11libs)
            libs.append("Xft")
            cflags = list(self.x11cflags)
            cflags.extend(self.ftcflags)
            ldflags = list(self.x11ldflags)
            ldflags.extend(self.ftldflags)
            self.disable_xft = not self._test_xft(self.x11inc, self.x11lib,
                                                  libs, cflags, ldflags)
        # -- Xt -------------------------------------
        if not self.disable_xt:
            libs = list(self.x11libs)
            libs.append("Xt")
            self.disable_xt = not self._test_xt(self.x11inc, self.x11lib, libs,
                                                self.x11cflags,
                                                self.x11ldflags)
        # -- gtk -------------------------------------
        if not self.disable_gtk:
            gtkcflags = None
            try:
                gtkcflags = shlex.split(Popen(["pkg-config", self.gtk_package,
                                               "--cflags"], stdout=PIPE,
                              stderr=PIPE).communicate()[0].rstrip())
            except OSError:
                pass
            if gtkcflags:
                self.gtkcflags = gtkcflags
                self.gtkldflags = shlex.split(Popen(["pkg-config",
                                     self.gtk_package, "--libs"], stdout=PIPE,
                                stderr=PIPE).communicate()[0].rstrip())
            else:
                self.disable_gtk = True
        # -- mupdf -------------------------------------
        if not self.disable_mupdf:
            for p in [os.path.join(os.sep, "usr", "local", "include"),
                      os.path.join(os.sep, "usr", "include")]:
                mupdf = os.path.join(p, "mupdf")
                if os.path.isdir(mupdf):
                    self.mupdfinc = [p]
                    break
            if not self.mupdfinc:
                self.disable_mupdf = True
            else:
                self.mupdflibs = ["mupdf",
                              "jbig2dec", "jpeg", "openjp2", "ssl", "crypto"]
                self.disable_mupdf = not self._test_mupdf(self.mupdfinc,
                                                          self.mupdflibs,
                                                  mupdfldflags=self.ftldflags)
                if self.disable_mupdf:
                    self.mupdfinc = []
                    self.mupdflibs = []
        # -- mov -------------------------------------
        if not self.disable_mov:
            self.disable_mov = not self._test_mov(self.mupdfinc, self.mupdflibs,
                                                  mupdfldflags=self.ftldflags)
        # -- opengl --------------------------------------------------
        if not self.disable_opengl:
            gllibs, glldflags = [], []
            if self.isDarwin:
                framework = ["-framework", "OpenGL", "-framework", "Cocoa"]
                glldflags.extend(framework)
            elif self.isLinux:
                gllibs.append("GL")
            self.disable_opengl = not self._test_opengl(gllibs=gllibs,
                                                        glldflags=glldflags)
            if not self.disable_opengl:
                self.gllibs, self.glldflags = gllibs, glldflags
        # -- platform extra link args -------------------------------------
        if self.isDarwin:
            self.platform_ldflags = ["-mmacosx-version-min=10.6",
                                     "-Wl,-rpath,@loader_path/."]
        elif self.isLinux:
            self.platform_ldflags = ["-Wl,-rpath,$ORIGIN"]

    def _finalize_Win32Pre(self):
        # -- wx -------------------------------------
        if not (self.wxdir and self.wxlib):
            self.disable_wx = True
        else:
            self.wxlibs = ["wxmsw29ud_core", "wxbase29ud"]
            self.wxinc = [os.path.join(self.wxdir, "include", "msvc"),
                          os.path.join(self.wxdir, "include")]
        # -- qt -------------------------------------
        if not self.disable_qt:
            # do not use _msvc_extra_link_args because /nodefaultlib causes
            # error LNK2019: unresolved external symbol ""__declspec(dllimport)
            #                void __cdecl std::_Xbad_alloc(void)" ...
            self.qtldflags = ["-dll"]
        # -- gs -------------------------------------
        if not self.disable_gs and self.gsdir:
            self.gsinc = [os.path.join(self.gsdir, "include")]
            self.gslib = [os.path.join(self.gsdir, "bin")]
            self.gslibs = ["gsdll32"]
            if (not os.path.isdir(self.gsinc[0]) or
                not os.path.isdir(self.gslib[0])):
                self.disable_gs = True
        # -- opengl --------------------------------------------------
        if not self.disable_opengl:
            self.gllibs.append("opengl32")
        # -- grdir -------------------------------------
        self.grdir = os.getenv("GRDIR", ("\\\"%s\\\""
                 % os.path.join(get_python_lib(plat_specific=True, prefix=''),
                                "gr").replace('\\', "\\\\")))

    def finalize_options(self):
        # -- platform finalizers (pre) -------------------------------------
        if self.isLinuxOrDarwin:
            self._finalize_LinuxOrDarwinPre()
        elif self.isWin32:
            self._finalize_Win32Pre()

        # -- platform independend finalizers -------------------------------
        # qt
        if not self.disable_qt:
            if not os.path.isfile(self.qmake):
                self.disable_qt = True
            else:
                qtversion = Popen([self.qmake, "-v"], stdout=PIPE,
                              stderr=STDOUT).communicate()[0].rstrip()
                match = re.search("\d\.\d\.\d", qtversion)
                if match:
                    self.qtversion = list(map(lambda s: int(s),
                                              match.group(0).split('.')))
                if not self.qtversion or self.qtversion[0] < 4:
                    self.disable_qt = True
                else:
                    qquery = Popen([self.qmake, "-query", "QT_INSTALL_HEADERS"],
                                    stdout=PIPE)
                    std = qquery.communicate()
                    if qquery.returncode == 0:
                        self.qtinc = [std[0].rstrip()]
                    qquery = Popen([self.qmake, "-query", "QT_INSTALL_LIBS"],
                                    stdout=PIPE)
                    std = qquery.communicate()
                    if qquery.returncode == 0:
                        self.qtlib = [std[0].rstrip()]

        # -- platform independend tests -------------------------------------
#        if not self.disable_gs:
#            self.disable_gs = not self._test_gs(self.gsinc, self.gslibs,
#                                                gsldflags=self.gsldflags)
#        if not self.disable_mupdf:
#            self.disable_mupdf = not self._test_mupdf(self.mupdfinc,
#                                                      self.mupdflibs,
#                                                  mupdfldflags=self.ftldflags)

        # -- platform finalizers (post) -------------------------------------
        if self.isLinuxOrDarwin:
            self._finalize_LinuxOrDarwinPost()
        elif self.isWin32:
            self._finalize_Win32Post()

        if self.disable_mupdf:
            self.disable_mov = True

        # disable freetype if Xft or Xt is not available
        if not self.disable_freetype:
            if self.disable_xft or self.disable_xt:
                self.disable_freetype = True

        # disable x11 if Xt is not available
        if not self.disable_x11:
            if self.disable_xt:
                self.disable_x11 = True

    def run(self):
        if self.isLinuxOrDarwin or self.isWin32:
            print(" isLinuxOrDarwin: ", self.isLinuxOrDarwin)
            print("         isLinux: ", self.isLinux)
            print("        isDarwin: ", self.isDarwin)
            print("         isWin32: ", self.isWin32)
            print("")
            print("          x11lib: ", self.x11lib)
            print("          x11inc: ", self.x11inc)
            print("         x11libs: ", self.x11libs)
            print("      x11ldflags: ", self.x11ldflags)
            print("       x11cflags: ", self.x11cflags)
            print("")
            print("        wxconfig: ", self.wxconfig)
            print("           wxdir: ", self.wxdir)
            print("           wxlib: ", self.wxlib)
            print("           wxinc: ", self.wxinc)
            print("          wxlibs: ", self.wxlibs)
            print("       wxldflags: ", self.wxldflags)
            print("           wxcxx: ", self.wxcxx)
            print("")
            print("      gtkldflags: ", self.gtkldflags)
            print("       gtkcflags: ", self.gtkcflags)
            print("")
            print("           qmake: ", self.qmake)
            print("           qtdir: ", self.qtdir)
            print("           qtinc: ", self.qtinc)
            print("           qtlib: ", self.qtlib)
            print("          qtlibs: ", self.qtlibs)
            print("       qtldflags: ", self.qtldflags)
            print("      Qt version: ", self.qtversion)
            print("")
            print("           gsdir: ", self.gsdir)
            print("           gsinc: ", self.gsinc)
            print("           gslib: ", self.gslib)
            print("          gslibs: ", self.gslibs)
            print("       gsldflags: ", self.gsldflags)
            print("")
            print("           grdir: ", self.grdir)
            print("")
            print(" freetype-config: ", self.ftconfig)
            print("       ftldflags: ", self.ftldflags)
            print("        ftcflags: ", self.ftcflags)
            print("")
            print("        mupdfinc: ", self.mupdfinc)
            print("       mupdflibs: ", self.mupdflibs)
            print("")
            print("      opengllibs: ", self.gllibs)
            print("    opengldflags: ", self.glldflags)
            print("")
            print("     disable-x11: ", self.disable_x11)
            print("      disable-xt: ", self.disable_xt)
            print("     disable-xft: ", self.disable_xft)
            print("      disable-wx: ", self.disable_wx)
            print("      disable-qt: ", self.disable_qt)
            print("     disable-gtk: ", self.disable_gtk)
            print("      disable-gs: ", self.disable_gs)
            print("     disable-fig: ", self.disable_fig)
            print("     disable-svg: ", self.disable_svg)
            print("    disable-html: ", self.disable_html)
            print("     disable-wmf: ", self.disable_wmf)
            print("     disable-mov: ", self.disable_mov)
            print("  disable-opengl: ", self.disable_opengl)
            print("  disable-quartz: ", self.disable_quartz)
            print("disable-freetype: ", self.disable_freetype)
            print("   disable-mupdf: ", self.disable_mupdf)
            print("")

            zinc = [os.path.join("3rdparty", "zlib")]
            defines = [("HAVE_ZLIB",), ("GRDIR", "\"%s\"" % self.grdir)]
            if self.disable_x11:
                defines.append(("NO_X11", 1))
            if self.disable_freetype:
                defines.append(("NO_FT", 1))
            if self.disable_mupdf:
                defines.append(("NO_MUPDF", 1))

            # -- GKS -------------------------------------
            inc = list(self.x11inc)
            inc.extend(zinc)
            lib = list(self.x11lib)
            libs = list(self.x11libs)
            ldflags = list(self.x11ldflags)
            ldflags.extend(self.ftldflags)
            cflags = list(self.x11cflags)
            cflags.extend(self.ftcflags)
            if self.isWin32:
                libs.extend(_libs_msvc)
                libs.append("msimg32")
                ldflags.append("/nodefaultlib")
                ldflags.append("-def:lib\gks\libgks.def")
                cflags.extend(_msvc_extra_compile_args)
            ldflags.extend(self.platform_ldflags)
            if self.isDarwin:
                cflags.append("-mmacosx-version-min=10.6")
                ldflags.append("-Wl,-install_name,@rpath/libGKS.so")
            if not self.disable_freetype:
                libs.append("fontconfig")
                libs.append("Xft")
            if not self.disable_freetype or not self.disable_xt:
                libs.append("Xt")
            gksExt = Extension("gr.libGKS", _gks_src_path,
                               define_macros=defines,
                               include_dirs=inc,
                               library_dirs=lib,
                               libraries=libs,
                               extra_link_args=ldflags,
                               extra_compile_args=cflags,
                               export_symbols=None)
            self.ext_modules.append(gksExt)

            # -- includes -------------------------------------
            # GKS include path (needed for GKS plugins)
            gksinc = [os.path.join("lib", "gks")]
            # 3rdparty include paths (needed for some GKS plugins)
            pnginc = [os.path.join("3rdparty", "png")]
            zinc = [os.path.join("3rdparty", "zlib")]
            jpeginc = [os.path.join("3rdparty", "jpeg")]

            ##
            # -- GKS plugins -------------------------------------

            # -- ghostscript -------------------------------------
            if not self.disable_gs:
                inc = list(self.gsinc)
                inc.extend(gksinc)
                lib = list(self.gslib)
                libs = list(self.gslibs)
                ldflags = list(self.gsldflags)
                cflags = []
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                gksGsExt = Extension("gr.gsplugin",
                                     _plugins_path["gsplugin.cxx"],
                                     define_macros=defines,
                                     include_dirs=inc,
                                     library_dirs=lib,
                                     libraries=libs,
                                     extra_link_args=ldflags,
                                     extra_compile_args=cflags)
                self.ext_modules.append(gksGsExt)

            # -- svg -------------------------------------
            if not self.disable_svg:
                inc = list(gksinc)
                inc.extend(pnginc)
                inc.extend(zinc)
                lib = [_build_3rdparty]
                libs = list(_pnglibs) # w32ERR: __imp_
                libs.extend(_zlibs)   # w32ERR: __imp_
                ldflags = []
                cflags = []
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                gksSvgExt = Extension("gr.svgplugin",
                                      _plugins_path["svgplugin.cxx"],
                                      define_macros=defines,
                                      include_dirs=inc,
                                      library_dirs=lib,
                                      libraries=libs,
                                      extra_link_args=ldflags,
                                      extra_compile_args=cflags)
                self.ext_modules.append(gksSvgExt)

            # -- htm -------------------------------------
            if not self.disable_html:
                inc = list(gksinc)
                inc.extend(pnginc)
                inc.extend(zinc)
                lib = [_build_3rdparty]
                libs = list(_pnglibs) # w32ERR: __imp_
                libs.extend(_zlibs)   # w32ERR: __imp_
                ldflags = []
                cflags = []
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                gksHtmExt = Extension("gr.htmplugin",
                                      _plugins_path["htmplugin.cxx"],
                                      define_macros=defines,
                                      include_dirs=inc,
                                      library_dirs=lib,
                                      libraries=libs,
                                      extra_link_args=ldflags,
                                      extra_compile_args=cflags)
                self.ext_modules.append(gksHtmExt)

            # -- fig -------------------------------------
            if not self.disable_fig:
                inc = list(gksinc)
                inc.extend(pnginc)
                inc.extend(zinc)
                lib = [_build_3rdparty]
                libs = list(_pnglibs) # w32ERR: __imp_
                libs.extend(_zlibs)   # w32ERR: __imp_
                ldflags = []
                cflags = []
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                gksFigExt = Extension("gr.figplugin",
                                      _plugins_path["figplugin.cxx"],
                                      define_macros=defines,
                                      include_dirs=inc,
                                      library_dirs=lib,
                                      libraries=libs,
                                      extra_link_args=ldflags,
                                      extra_compile_args=cflags)
                self.ext_modules.append(gksFigExt)

            # -- wmf -------------------------------------
            if not self.disable_wmf:
                inc = list(gksinc)
                libs = []
                ldflags = []
                cflags = []
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                gksWmfExt = Extension("gr.wmfplugin",
                                      _plugins_path["wmfplugin.cxx"],
                                      define_macros=defines,
                                      include_dirs=inc,
                                      libraries=libs,
                                      extra_link_args=ldflags,
                                      extra_compile_args=cflags)
                self.ext_modules.append(gksWmfExt)

            # -- mov -------------------------------------
            if not self.disable_mov:
                inc = list(self.mupdfinc)
                inc.extend(gksinc)
                libs = list(self.mupdflibs)
                ffmpeglibs = ["avdevice", "avformat", "avfilter", "avcodec",
                              "swscale", "avutil"]
                if self.isDarwin:
                    ffmpeglibs.extend(["theora", "ogg", "pvx"])
                libs.extend(_zlibs)
                libs.append("pthread")
                ldflags = list(self.ftldflags)
                cflags = list(self.ftcflags)
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
#                    ldflags.extend(_msvc_extra_link_args)   # necessary?
                    cflags.extend(_msvc_extra_compile_args)
                gksMovExt = Extension("gr.movplugin",
                                      _plugins_path["movplugin.cxx"],
                                      define_macros=defines,
                                      include_dirs=inc,
                                      libraries=libs,
                                      extra_link_args=ldflags,
                                      extra_compile_args=cflags)
                self.ext_modules.append(gksMovExt)

            # -- wx -------------------------------------
            if not self.disable_wx:
                inc = list(self.wxinc)
                inc.extend(gksinc)
                lib = list(self.wxlib)
                libs = list(self.wxlibs)
                ldflags = list(self.wxldflags)
                cflags = list(self.wxcxx)
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                    cflags.append("/DWXUSINGDLL")
                    cflags.append("/DwxMSVC_VERSION_AUTO")
                    cflags.append("/D_UNICODE")
                gksWxExt = Extension("gr.wxplugin",
                                     _plugins_path["wxplugin.cxx"],
                                     define_macros=defines,
                                     include_dirs=inc,
                                     library_dirs=lib,
                                     libraries=libs,
                                     extra_link_args=ldflags,
                                     extra_compile_args=cflags)
                self.ext_modules.append(gksWxExt)

            # -- qt -------------------------------------
            if not self.disable_qt:
                inc = list(self.qtinc)
                inc.extend(gksinc)
                lib = list(self.qtlib)
                libs = list(self.qtlibs)
                ldflags = list(self.qtldflags)
                cflags = []
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
# do not use _msvc_extra_link_args because /nodefaultlib causes
# error LNK2019: unresolved external symbol ""__declspec(dllimport)
#                void __cdecl std::_Xbad_alloc(void)" ...
                    ldflags.append("-dll")
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                gksQtExt = Extension("gr.qtplugin",
                                     _plugins_path["qtplugin.cxx"],
                                     define_macros=defines,
                                     include_dirs=inc,
                                     library_dirs=lib,
                                     libraries=libs,
                                     extra_link_args=ldflags,
                                     extra_compile_args=cflags)
                self.ext_modules.append(gksQtExt)

            # -- gtk -------------------------------------
            if not self.disable_gtk:
                inc = list(gksinc)
#                inc.extend(self.x11inc)
                ldflags = list(self.gtkldflags)
                cflags = list(self.gtkcflags)
                ldflags.extend(self.platform_ldflags)
                if self.isWin32:
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                gksGtkExt = Extension("gr.gtkplugin",
                                      _plugins_path["gtkplugin.cxx"],
                                      define_macros=defines,
                                      include_dirs=inc,
                                      extra_compile_args=cflags,
                                      extra_link_args=ldflags)
                self.ext_modules.append(gksGtkExt)

            # -- quartz -------------------------------------
            if not self.disable_quartz:
                inc = list(gksinc)
                libs = ["objc"]
                ldflags = ["-framework", "Foundation",
                           "-framework", "ApplicationServices",
                           "-framework", "AppKit"]
                ldflags.extend(self.platform_ldflags)
                gksQuartzExt = Extension("gr.quartzplugin",
                                         _plugins_path["quartzplugin.m"],
                                         define_macros=defines,
                                         include_dirs=inc,
                                         libraries=libs,
                                         extra_link_args=ldflags)
                self.ext_modules.append(gksQuartzExt)

            # -- GR -------------------------------------
            inc = list(gksinc)
            inc.extend(zinc)
            inc.extend(pnginc)
            inc.extend(jpeginc)
            inc.extend(self.mupdfinc)
            inc.extend(self.x11inc) # at least because this includes wrong png.h
            lib = list(self.x11lib)
            lib.append(_build_lib_grpkg)
            lib.append(_build_3rdparty)
            if self.isLinuxOrDarwin:
                libs = ["GKS"]
            else:
                libs = ["libGKS"]
                lib.append(os.path.join(_build_temp, "Release", "lib", "gks"))
            libs.extend(self.x11libs)
            libs.extend(self.mupdflibs)
            ldflags = list(self.x11ldflags)
            ldflags.extend(self.ftldflags)
            # important: lib ordering png, jpeg, z
            staticlibs = [_libpng, _libjpeg, _libz]
            ldflags.extend(staticlibs)
            cflags = list(self.x11cflags)
            cflags.extend(self.ftcflags)
            if self.isWin32:
                libs.extend(_libs_msvc)
                cflags.extend(_msvc_extra_compile_args)
#                ldflags.extend(_msvc_extra_link_args)
            ldflags.extend(self.platform_ldflags)
            if self.isDarwin:
                ldflags.append("-Wl,-install_name,@rpath/libGR.so")
            grExt = Extension("gr.libGR", _gr_src_path,
                              define_macros=defines,
                              include_dirs=inc,
                              library_dirs=lib,
                              libraries=libs,
                              extra_link_args=ldflags,
                              extra_compile_args=cflags)
            self.ext_modules.append(grExt)
            grinc = inc
            grlib = lib
            grlibs = libs

            # -- GR3 -------------------------------------
            if not self.disable_opengl:
                inc = list(grinc)
                inc.append(os.path.join("lib", "gr"))
                lib = list(grlib)
                libs = list(grlibs)
                libs.extend(self.gllibs)
                ldflags = list(self.x11ldflags)
                ldflags.extend(self.ftldflags)
                ldflags.extend(self.platform_ldflags)
                ldflags.extend(self.glldflags)
                # important: lib ordering png, jpeg, z
                staticlibs = [_libpng, _libjpeg, _libz]
                ldflags.extend(staticlibs)
                if self.isLinuxOrDarwin:
                    libs.append("GR")
                if self.isDarwin:
                    ldflags.append("-Wl,-install_name,@rpath/libGR3.so")
                    ldflags.append("-Wl,-rpath,@loader_path/../gr/.")
                elif self.isLinux:
                    ldflags.append("-Wl,-rpath,$ORIGIN/../gr")
                elif self.isWin32:
                    inc.append("3rdparty")
                    lib.append(os.path.join(_build_temp, "Release", "lib",
                                            "gks"))
                    lib.append(os.path.join(_build_temp, "Release", "lib",
                                            "gr"))
                    libs.append("libGR")
                    libs.extend(_libs_msvc)
                    cflags.extend(_msvc_extra_compile_args)
                    ldflags.append("-dll")
                gr3Ext = Extension("gr3.libGR3", _gr3_src_path,
                                   define_macros=defines,
                                   include_dirs=inc,
                                   library_dirs=lib,
                                   libraries=libs,
                                   extra_link_args=ldflags,
                                   extra_compile_args=cflags)
                self.ext_modules.append(gr3Ext)
        else:
            print("Platform \"", sys.platform, "\" is not supported.",
                  file=sys.stderr)
            sys.exit(-3)


class build_ext(_build_ext, check_ext, build_static):

    # workaround for libGKS on win32 no "init" + module_name function
    def get_export_symbols (self, ext):
        """Return the list of symbols that a shared extension has to
        export. This overloaded function does not append "init" + module_name
        to the exported symbols (default in superclass function).

        """
        return ext.export_symbols

    # workaround do not use pyd files in this project because we have our
    # own ctypes wrapper. Use dll files instead.
    def get_ext_filename(self, ext_name):
        r"""Convert the name of an extension (eg. "foo.bar") into the name
        of the file from which it will be loaded (eg. "foo/bar.so", or
        "foo\bar.dll not foo\bar.pyd (default in superclass function)").

        """
        ret = _build_ext.get_ext_filename(self, ext_name)
        (pathNoExt, ext) = os.path.splitext(ret)
        if ext == ".pyd":
            ret = pathNoExt + ".dll"
        return ret

    def initialize_options(self):
        check_ext.initialize_options(self)
        build_static.initialize_options(self)
        _build_ext.initialize_options(self)

    def finalize_options(self):
        check_ext.finalize_options(self)
        build_static.initialize_options(self)
        _build_ext.finalize_options(self)
        self.extensions = self.ext_modules

    def run(self):
        check_ext.run(self)
        build_static.run(self)
        _build_ext.run(self)
        if not self.disable_quartz:
            os.system("xcodebuild -project lib/gks/quartz/GKSTerm.xcodeproj")
            os.system("ditto lib/gks/quartz/build/Release/GKSTerm.app " +
                      os.path.join(_build_scripts, "GKSTerm.app"))


# check wether this is a Debian based system, e.g. Ubuntu
if "deb_system" in distutils.command.install.INSTALL_SCHEMES:
    # Overwrite all site-packages paths with dist-packages in INSTALL_SCHEMES
    # on Debian based systems, e.g. Ubuntu, because site-packages is not in
    # sys.path (PYTHONPATH) per default.
    # Without this patch installation with prefix shows following
    # unexpected behavior:
    # - packages will be installed into dist-packages
    # - data_files will be installed into site-packages (!)
    #   instead of dist-packages (as expected).
    # Installation without prefix works as expected.
    INSTALL_SCHEMES = distutils.command.install.INSTALL_SCHEMES
    for k, d in INSTALL_SCHEMES.items():
        for kk, v in d.items():
            INSTALL_SCHEMES[k][kk] = v.replace("site-packages", "dist-packages")

# unique platform id used by distutils
_uPlatformId = "%s-%d.%d" % (get_platform(), sys.version_info[0],
                             sys.version_info[1])
_build_lib = os.path.join("build", "lib." + _uPlatformId)
_build_lib_grpkg = os.path.join(_build_lib, "gr")
_build_3rdparty = os.path.join("build", "3rdparty." + _uPlatformId)
_build_temp = os.path.join("build", "temp." + _uPlatformId)
_build_scripts = os.path.join("build", "scripts-%d.%d" % (sys.version_info[0],
                                                          sys.version_info[1]))

# additional data files in the distribtion
_gks_fonts = os.path.join("lib", "gks", "fonts")
_gks_fonts_path = list(map(lambda f: os.path.join(_gks_fonts, f),
                           os.listdir(_gks_fonts)))
_gks_fonts_path.append(os.path.join("lib", "gks", "gksfont.dat"))
_data_files = [(os.path.join(get_python_lib(plat_specific=True, prefix=''),
                             "gr", "fonts"), _gks_fonts_path)]

# prerequisites: static 3rdparty libraries
if sys.platform == "win32":
    # if linking against png, jpeg and zlib use this ordering!
    _libpng = os.path.join(_build_3rdparty, "png.lib")
    _libjpeg = os.path.join(_build_3rdparty, "jpeg.lib")
    _libz = os.path.join(_build_3rdparty, "z.lib")

    _libs_msvc = ["msvcrt", "oldnames", "kernel32", "wsock32", "advapi32",
                  "user32", "gdi32", "comdlg32", "winspool"]
    # w32ERR: __imp_ -> fixed by linking against oldnames.lib
    _msvc_extra_compile_args = ["/Zi", "/D_DLL", "/D_POSIX", "/nologo"]
    _msvc_extra_link_args = ["/nodefaultlib", "-dll"]
else:
    _libz = os.path.join(_build_3rdparty, "libz.a")
    _libpng = os.path.join(_build_3rdparty, "libpng.a")
    _libjpeg = os.path.join(_build_3rdparty, "libjpeg.a")

    _libs_msvc = []
    _msvc_extra_compile_args = []
    _msvc_extra_link_args = []

_gks_src = ["gks.c", "gksforbnd.c", "font.c", "afm.c", "util.c", "ft.c", "dl.c",
            "malloc.c", "error.c", "mf.c", "wiss.c", "cgm.c", "win.c", "mac.c",
            "ps.c", "pdf.c", "x11.c", "socket.c", "plugin.c", "compress.c",
            "io.c"]
_gks_plugin_src = ["font.cxx", "afm.cxx", "util.cxx", "dl.cxx",
                   "malloc.cxx", "error.cxx", "io.cxx"]
_gks_plugins = ["wxplugin.cxx", "qtplugin.cxx", "gtkplugin.cxx",
                "quartzplugin.m", "svgplugin.cxx", "figplugin.cxx",
                "gsplugin.cxx", "wmfplugin.cxx", "movplugin.cxx",
                "htmplugin.cxx"]

_libz_src = ["adler32.c", "compress.c", "crc32.c", "deflate.c", "gzclose.c",
             "gzlib.c", "gzread.c", "gzwrite.c", "infback.c", "inffast.c",
             "inflate.c", "inftrees.c", "trees.c", "uncompr.c", "zutil.c"]

_libpng_src = ["png.c", "pngerror.c", "pngget.c", "pngmem.c", "pngpread.c",
               "pngread.c", "pngrio.c", "pngrtran.c", "pngrutil.c", "pngset.c",
               "pngtrans.c", "pngwio.c", "pngwrite.c", "pngwtran.c",
               "pngwutil.c"]

_libjpeg_src = ["jaricom.c", "jcapimin.c", "jcapistd.c", "jcarith.c",
                "jccoefct.c", "jccolor.c", "jcdctmgr.c", "jchuff.c", "jcinit.c",
                "jcmainct.c", "jcmarker.c", "jcmaster.c", "jcomapi.c",
                "jcparam.c", "jcprepct.c", "jcsample.c", "jctrans.c",
                "jdapimin.c", "jdapistd.c", "jdarith.c", "jdatadst.c",
                "jdatasrc.c", "jdcoefct.c", "jdcolor.c", "jddctmgr.c",
                "jdhuff.c", "jdinput.c", "jdmainct.c", "jdmarker.c",
                "jdmaster.c", "jdmerge.c", "jdpostct.c", "jdsample.c",
                "jdtrans.c", "jerror.c", "jfdctflt.c", "jfdctfst.c",
                "jfdctint.c", "jidctflt.c", "jidctfst.c", "jidctint.c",
                "jmemmgr.c", "jmemnobs.c", "jquant1.c", "jquant2.c", "jutils.c"]

_gr_src = ["gr.c", "text.c", "contour.c", "spline.c", "gridit.c", "strlib.c",
           "io.c", "image.c", "md5.c", "import.c", "grforbnd.c"]

_gr3_src = ["gr3.c", "gr3_convenience.c", "gr3_html.c", "gr3_povray.c",
            "gr3_png.c", "gr3_jpeg.c", "gr3_gr.c", "gr3_mc.c"]

if sys.platform == "darwin":
    os.environ["ARCHFLAGS"] = os.getenv("ARCHFLAGS", "-arch x86_64")
    os.environ["LDSHARED"] = get_config_var("LDSHARED").replace("-bundle",
                                                                "-dynamiclib")
    _gr3_src.insert(0, "gr3_cgl.c")
elif "linux" in sys.platform:
    _gr3_src.insert(0, "gr3_glx.c")
elif sys.platform == "win32":
    _gr3_src.insert(0, "gr3_win.c")

_gks_src_path = list(map(lambda p: os.path.join("lib", "gks", p), _gks_src))
_gks_plugin_src_path = list(map(lambda p: os.path.join("lib", "gks", "plugin",
                                                       p), _gks_plugin_src))
_libz_src_path = list(map(lambda p: os.path.join("3rdparty", "zlib", p),
                          _libz_src))
_libpng_src_path = list(map(lambda p: os.path.join("3rdparty", "png", p),
                            _libpng_src))
_libjpeg_src_path = list(map(lambda p: os.path.join("3rdparty", "jpeg", p),
                             _libjpeg_src))
_gr_src_path = list(map(lambda p: os.path.join("lib", "gr", p), _gr_src))
_gr3_src_path = list(map(lambda p: os.path.join("lib", "gr3", p), _gr3_src))

_plugins_path = {}
for plugin_src in _gks_plugins:
    _plugins_path[plugin_src] = list(_gks_plugin_src_path)
    _plugins_path[plugin_src].append(os.path.join("lib", "gks", "plugin",
                                                  plugin_src))

_zlibs = ["z"]
_pnglibs = ["png"]
_jpeglibs = ["jpeg"]


# -- setup -----------------------------------------------------------------
# [OSX] Install GSKTerm.app.
if sys.platform == "darwin":
    _scripts = [os.path.join(_build_scripts, "GKSTerm.app")]
else:
    _scripts = None

setup(cmdclass={"build_ext": build_ext, "check_ext": check_ext,
                "build_static": build_static, "clean_static": clean_static,
                "clean": clean},
      name="gr",
      version=__version__,
      description="Python visualization framework",
      long_description="""
        GR is a universal framework for cross-platform visualization
        applications. It offers developers a compact, portable and consistent
        graphics library for their programs. Applications range from
        publication quality 2D graphs to the representation of complex 3D
        scenes.
        """,
      author="Scientific IT-Systems",
      author_email="j.heinen@fz-juelich.de",
      license="GNU General Public License",
      url="http://gr-framework.org",
      package_dir={'': "lib/gr/python",
                   "gr3": "lib/gr3/python/gr3"},
      requires=["numpy"],
      packages=["gr", "gr.pygr", "gr.matplotlib", "gr3",
                "qtgr", "qtgr.events"],
      # ext_modules dummy entry
      # check_ext dynamically generates a list of Extensions
      ext_modules=[Extension("", [""])],
      scripts=_scripts,
      data_files=_data_files)



# -*- coding: utf-8 -*-
"""Python GR Helper Classes

Exported Classes:

"""
# standard library
import math
# local library
from gr._version import __version__, __revision__

__author__ = "Christian Felder <c.felder@fz-juelich.de>"
__copyright__ = """Copyright 2012 - 2014 Forschungszentrum Juelich GmbH

This file is part of GR, a universal framework for visualization applications.
Visit http://gr-framework.org for the latest
version.

GR was developed by the Scientific IT-Systems group at the Peter Gr�nberg
Institute at Forschunsgzentrum J�lich. The main development has been done
by Josef Heinen who currently maintains the software.

GR is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GR. If not, see <http://www.gnu.org/licenses/>.
 
"""


class DomainChecker(object):

    _ZERO = 1e-8
    _EPSILON = 1e-8

    @staticmethod
    def isInLogDomain(*args):
        res = True
        for value in args:
            if value <= DomainChecker._ZERO:
                res = False
                break
        return res

    @staticmethod
    def isInWindowDomain(xmin, xmax, ymin, ymax):
        res = True
        if (math.isnan(xmin) or math.isinf(xmin) or math.isnan(xmax)
            or math.isinf(xmax) or math.isnan(ymin) or math.isinf(ymin) or
            math.isnan(ymax) or math.isinf(ymax) or
            xmin > max or ymin > ymax or abs(xmax - xmin) < DomainChecker._EPSILON or
            abs(ymax - ymin) < DomainChecker._EPSILON):
            res = False
        return res


class ColorIndexGenerator(object):

    _distinct_colors = range(980, 1000)
    _n = len(_distinct_colors)
    _curIdx = 0

    def __init__(self):
        self._i = 0

    @staticmethod
    def nextColorIndex():
        idx = ColorIndexGenerator._distinct_colors[ColorIndexGenerator._curIdx]
        ColorIndexGenerator._curIdx = ((ColorIndexGenerator._curIdx + 1)
                                       % ColorIndexGenerator._n)
        return idx

    def getNextColorIndex(self):
        idx = ColorIndexGenerator._distinct_colors[self._i]
        self._i = (self._i + 1) % ColorIndexGenerator._n
        return idx

    def reset(self):
        self._i = 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
GR / PyQt4 interoperability example
"""

import sys
import os
from PyQt4 import QtCore, QtGui
import sip
from gr.pygr import *

class GrWidget(QtGui.QWidget) :
    def __init__(self, *args) :
        QtGui.QWidget.__init__(self)
        
        self.setupUi(self)

        os.environ["GKS_WSTYPE"] = "381"
        os.environ["GKS_DOUBLE_BUF"] = "True"

        self.connect(self.DrawButton, QtCore.SIGNAL("clicked()"), self.draw)
        self.connect(self.QuitButton, QtCore.SIGNAL("clicked()"), self.quit)
        self.w = 500
        self.h = 500
        self.sizex = 1
        self.sizey = 1
 
    def setupUi(self, Form) :

        Form.setWindowTitle("GrWidget")
        Form.resize(QtCore.QSize(500, 500).expandedTo(Form.minimumSizeHint()))

        self.DrawButton = QtGui.QPushButton(Form)
        self.DrawButton.setText("Draw")
        self.DrawButton.setGeometry(QtCore.QRect(290, 5, 100, 25))
        self.DrawButton.setObjectName("draw")

        self.QuitButton = QtGui.QPushButton(Form)
        self.QuitButton.setText("Quit")
        self.QuitButton.setGeometry(QtCore.QRect(395, 5, 100, 25))
        self.QuitButton.setObjectName("quit")

        QtCore.QMetaObject.connectSlotsByName(Form)

    def quit(self) :
        gr.emergencyclosegks()
        self.close()

    def draw(self) :
        self.setStyleSheet("background-color:white;");

        x = range(0, 128)
        y = range(0, 128)
        z = readfile(os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                  "kws.dat"), separator='$')
        zrange = max(z) - min(z)
        h = [min(z) + i * 0.025 * zrange for i in range(0, 40)]

        gr.clearws()
        mwidth  = self.w * 2.54 / self.logicalDpiX() / 100
        mheight = self.h * 2.54 / self.logicalDpiY() / 100
        gr.setwsviewport(0, mwidth, 0, mheight)
        gr.setwswindow(0, self.sizex, 0, self.sizey)
        gr.setviewport(0.075 * self.sizex, 0.95 * self.sizex, 0.075 * self.sizey, 0.95 * self.sizey)
        gr.setwindow(1, 128, 1, 128)
        gr.setspace(min(z), max(z), 0, 90)
        gr.setcharheight(0.018)
        gr.setcolormap(-3)
        gr.surface(x, y, z, 5)
        gr.contour(x, y, h, z, -1)
        gr.axes(5, 5, 1, 1, 2, 2, 0.0075)
        self.update()

    def resizeEvent(self, event):
        self.w = event.size().width()
        self.h = event.size().height()
        if self.w > self.h:
          self.sizex = 1
          self.sizey = float(self.h)/self.w
        else:
          self.sizex = float(self.w)/self.h
          self.sizey = 1
        self.draw()

    def paintEvent(self, ev) :
        self.painter = QtGui.QPainter()
        self.painter.begin(self)
        self.painter.drawText(15, 15, "Contour Example using PyQt4 ...")
        os.environ['GKSconid'] = "%x!%x" % (sip.unwrapinstance(self), sip.unwrapinstance(self.painter))
        gr.updatews()
        self.painter.end()

app = QtGui.QApplication(sys.argv)
if not sys.platform == "linux2" :
    app.setStyle('Windows')

w = GrWidget()
w.show()

sys.exit(app.exec_())



#!/usr/bin/env python
# -*- no-plot -*-
"""
Compare figure output performance of Matplotlib vs. GR
"""

from numpy import arange, sin, pi
from time import time
from os import environ
from sys import argv, stdout

if len(argv) > 1:
    dev = argv[1]
else:
    dev = 'ps'

import matplotlib
matplotlib.use(dev)

from pylab import clf, plot, draw, savefig

x = arange(0, 2 * pi, 0.01)

tstart = time()
for i in arange(1, 100):
    clf()
    plot(sin(x + i / 10.0))
    savefig('mpl%04d.%s' % (i, dev))
    if i % 2 == 0:
        print '\b.',
        stdout.flush()


fps_mpl = int(100 / (time() - tstart))
print('fps (mpl): %4d' % fps_mpl)

from gr.pygr import plot

tstart = time() 
environ["GKS_WSTYPE"] = dev
for i in arange(1, 100):
    plot(x, sin(x + i / 10.0))
    if i % 2 == 0:
        print '\b.',
        stdout.flush()

fps_gr = int(100 / (time() - tstart))
print('fps  (GR): %4d' % fps_gr)

print('  speedup: %6.1f' % (float(fps_gr) / fps_mpl))



import sys
import socket
import bottle

from struct import unpack
from thread import start_new_thread
from gevent import monkey

import parser

monkey.patch_all()

plots = []
pindex = 0

HOST = ''
PORT = 8410

def listen():
    global plots

    sock_stream = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        sock_stream.bind((HOST, PORT))
    except socket.error, msg:
        print('Bind failed: ' + msg[1])
        sys.exit()

    sock_stream.listen(5)

    while True:
        conn, addr = sock_stream.accept()
        while True:
            data = conn.recv(4)
            if not data:
                break
            length = unpack('i', data)[0]
            data = ''
            while len(data) < length:
                chunk = conn.recv(length - len(data))
                if chunk == '':
                    break
                data += chunk
            if len(data) != length:
                break
            plots.append(data.decode('latin-1').encode('utf-8'))
        conn.close()

@bottle.get('/stream')
def sendplot():
    global plots
    bottle.response.content_type  = 'text/event-stream'
    bottle.response.cache_control = 'no-cache'

    # Set client-side auto-reconnect timeout, ms.
    yield 'retry: 100\n\n'

    while True:
        if len(plots) > 0:
            yield 'data: %s\n\n' % plots.pop().replace('\n','')
        else:
            yield ''

@bottle.route('/<filename>')
def send_static(filename='index.html'):
    '''
    return the requested static web page to the web browser
    '''
    return bottle.static_file(filename, root='./')

@bottle.route('/')
def index():
    return bottle.static_file('index.html', root='./')

start_new_thread(listen, ())

bottle.run(server=bottle.GeventServer, host='127.0.0.1', port=8080)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# local library
import gr
from gr.pygr import Plot, PlotAxes, PlotCurve, Text

if __name__ == '__main__':
    tx, ty = 0, -20
    x = [-3.3 + t * .1 for t in range(66)]
    y = [t ** 5 - 13 * t ** 3 + 36 * t for t in x]
    txtfmt = "Text drawn on\n(%g, %g) with\nhalign left, valign top"

    plt = Plot((.1, .95, .1, .88))
    plt.title = "Text on Axes Example"
    plt.subTitle ="Show usage of gr.pygr.Text"
    plt.xlabel = "x"
    plt.ylabel = "y"

    curve = PlotCurve(x, y, legend="foo bar")
    axes = PlotAxes(plt.viewport).addCurves(curve)
    axes.setWindow(-4.0, 4.0, -60.0, 40.0)
    text = Text(tx, -ty, txtfmt % (tx, -ty), axes, .02)
    plt.addAxes(axes)
    text2 = Text(tx, ty, txtfmt % (tx, ty), axes, .02)
    tbx, tby = text2.getBoundingBox()

    plt.drawGR()
    text.drawGR()
    text2.drawGR()

    # set viewport and window accordingly to draw in NDC space
    gr.setviewport(0, axes.sizex, 0, axes.sizey)
    gr.setwindow(0, axes.sizex, 0, axes.sizey)
    gr.fillarea(tbx, tby)



#!/usr/bin/env python
# -*- no-plot -*-
"""
3D surface plot from neutron spectrum data
"""

import os
from gr.pygr import *

os.environ["GR_DISPLAY"] = "localhost:"
os.system("glgr")
delay(2)

counts = readfile(os.path.join(os.path.dirname(os.path.realpath(__file__)),
                               "kws2.dat"),
                  separator='$')
plot3d(counts,
       rotation=45, tilt=30,
       colormap=4, contours=False,
       xtitle='X',
       ytitle='Y',
       ztitle="Counts")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Qt backend GR module

The default backend order (PySide, PyQt4) can be overwritten with:
    setattr(sys, "QT_BACKEND_ORDER", ["PyQt4", "PySide"])
"""

import sys
# local library
from gr._version import __version__, __revision__

__author__ = "Christian Felder <c.felder@fz-juelich.de>"
__copyright__ = """Copyright 2012-2014 Forschungszentrum Juelich GmbH

This file is part of GR, a universal framework for visualization applications.
Visit http://gr-framework.org for the latest
version.

GR was developed by the Scientific IT-Systems group at the Peter Grünberg
Institute at Forschunsgzentrum Jülich. The main development has been done
by Josef Heinen who currently maintains the software.

GR is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GR. If not, see <http://www.gnu.org/licenses/>.
 
"""

QT_PYSIDE = "PySide"
QT_PYQT4 = "PyQt4"
QT_BACKEND_ORDER = getattr(sys, "QT_BACKEND_ORDER", [QT_PYSIDE, QT_PYQT4])

del sys



#!/usr/bin/env python
# -*- animation -*-
"""
3D animation of a double pendulum
"""

from numpy import sin, cos, pi, array
import time
import gr
import gr3

g = 9.8        # gravitational constant

def rk4(x, h, y, f):
    k1 = h * f(x, y)
    k2 = h * f(x + 0.5 * h, y + 0.5 * k1)
    k3 = h * f(x + 0.5 * h, y + 0.5 * k2)
    k4 = h * f(x + h, y + k3)
    return x + h, y + (k1 + 2 * (k2 + k3) + k4) / 6.0

def pendulum_derivs(t, state):
    # The following derivation is from:
    # http://scienceworld.wolfram.com/physics/DoublePendulum.html
    t1, w1, t2, w2 = state
    a = (m1 + m2) * l1
    b = m2 * l2 * cos(t1 - t2)
    c = m2 * l1 * cos(t1 - t2)
    d = m2 * l2
    e = -m2 * l2 * w2**2 * sin(t1 - t2) - g * (m1 + m2) * sin(t1)
    f =  m2 * l1 * w1**2 * sin(t1 - t2) - m2 * g * sin(t2)
    return array([w1, (e*d-b*f) / (a*d-c*b), w2, (a*f-c*e) / (a*d-c*b)])

def double_pendulum(theta, length, mass):
    gr.clearws()
    gr.setviewport(0, 1, 0, 1)
    
    direction = []
    position = [(0, 0, 0)]
    for i in range(2):
        direction.append((sin(theta[i]) * length[i] * 2,
                          -cos(theta[i]) * length[i] * 2, 0))
        position.append([position[-1][j] + direction[-1][j] for j in range(3)])
    
    gr3.clear()
    # draw pivot point
    gr3.drawcylindermesh(1, (0, 0.2, 0), (0, 1, 0), (0.4, 0.4, 0.4), 0.4, 0.05)
    gr3.drawcylindermesh(1, (0, 0.2, 0), (0, -1, 0), (0.4, 0.4, 0.4), 0.05, 0.2)
    gr3.drawspheremesh(1, (0,0,0), (0.4, 0.4, 0.4), 0.05)
    # draw rods
    gr3.drawcylindermesh(2, position, direction,
                         (0.6, 0.6, 0.6) * 2, (0.05, 0.05),
                         [l * 2 for l in length])
    # draw bobs
    gr3.drawspheremesh(2, position[1:], (1, 1, 1) * 2, [m * 0.2 for m in mass]) 

    gr3.drawimage(0, 1, 0, 1, 500, 500, gr3.GR3_Drawable.GR3_DRAWABLE_GKS)
    gr.updatews()
    return

l1 = 1.2       # length of rods
l2 = 1.0
m1 = 1.0       # weights of bobs
m2 = 1.5
t1 = 100.0     # inintial angles
t2 = -20.0

w1 = 0.0
w2 = 0.0
t = 0
dt = 0.04
state = array([t1, w1, t2, w2]) * pi / 180

gr3.init()
gr3.setcameraprojectionparameters(45, 1, 100)
gr3.cameralookat(6, -2, 4, 0, -2, 0, 0, 1, 0)
gr3.setbackgroundcolor(1, 1, 1, 1)
gr3.setlightdirection(1, 1, 10)

now = time.clock()

while t < 30:
    start = now

    t, state = rk4(t, dt, state, pendulum_derivs)
    t1, w1, t2, w2 = state
    double_pendulum([t1, t2], [l1, l2], [m1, m2])

    now = time.clock()
    if start + dt > now:
        time.sleep(start + dt - now)




__all__ = ['GR3_InitAttribute',
           'GR3_Error',
           'GR3_Exception',
           'GR3_Quality',
           'GR3_Drawable',
           'init',
           'terminate',
           'getimage',
           'export',
           'getrenderpathstring',
           'setlogcallback',
           'drawimage',
           'createmesh',
           'createindexedmesh',
           'drawmesh',
           'createheightmapmesh',
           'drawheightmap',
           'deletemesh',
           'setquality',
           'clear',
           'cameralookat',
           'setcameraprojectionparameters',
           'setlightdirection',
           'drawcylindermesh',
           'drawconemesh',
           'drawspheremesh',
           'drawcubemesh',
           'setbackgroundcolor',
           'triangulate',
           'createisosurfacemesh']


import sys
if any([module.startswith('OpenGL') for module in sys.modules]):
    import warnings
    warnings.warn("Importing gr3 after importing pyOpenGL (or any of its modules) might cause problems on some platforms. Please import gr3 first to avoid this.")

from sys import version_info, platform
from platform import python_implementation

from ctypes import c_int, c_uint, c_ushort, c_ubyte, c_float, c_double, \
                   c_char, c_char_p, c_ulong, py_object, POINTER, byref, \
                   CFUNCTYPE, CDLL, create_string_buffer, cast
import ctypes.util
import numpy
import os
import gr


_impl = python_implementation()
if _impl != 'PyPy':
    from ctypes import pythonapi

_gr3PkgDir = os.path.realpath(os.path.dirname(__file__))
_gr3LibDir = os.getenv("GR3LIB", _gr3PkgDir)
if sys.platform == "win32":
    libext = ".dll"
else:
    libext = ".so"

# Detect whether this is a site-package installation
if os.access(os.path.join(_gr3PkgDir, "libGR3" + libext), os.R_OK):
    # Set GRDIR environment accordingly to site-package installation.
    # (needed for finding GKSTerm on OSX)
    os.environ["GRDIR"] = os.getenv("GRDIR",
        os.path.join(os.path.realpath(os.path.dirname(__file__)), "..", "gr"))

_gr3Lib = os.path.join(_gr3LibDir, "libGR3" + libext)
if not os.getenv("GR3LIB") and not os.access(_gr3Lib, os.R_OK):
    _gr3LibDir = os.path.join(_gr3PkgDir, "..", "..")
    _gr3Lib = os.path.join(_gr3LibDir, "libGR3" + libext)
_gr3 = CDLL(_gr3Lib)

class intarray:
    def __init__(self, a):
        if _impl == 'PyPy':
            self.array = numpy.array(a, numpy.int32)
            self.data = cast(self.array.__array_interface__['data'][0],
                             POINTER(c_int))
        else:
            self.array = numpy.array(a, c_int)
            self.data = self.array.ctypes.data_as(POINTER(c_int))

class floatarray:
    def __init__(self, a, copy=False):
        if _impl == 'PyPy':
            self.array = numpy.array(a, numpy.float32, copy=copy)
            self.data = cast(self.array.__array_interface__['data'][0],
                             POINTER(c_float))
        else:
            self.array = numpy.array(a, c_float)
            self.data = self.array.ctypes.data_as(POINTER(c_float))

def char(string):
    if version_info[0] == 3:
        s = create_string_buffer(string.encode('iso8859-15'))
    else:
        s = create_string_buffer(string)
    return cast(s, c_char_p)

class GR3_InitAttribute(object):
    GR3_IA_END_OF_LIST = 0
    GR3_IA_FRAMEBUFFER_WIDTH = 1
    GR3_IA_FRAMEBUFFER_HEIGHT = 2

class GR3_Error(object):
    GR3_ERROR_NONE = 0
    GR3_ERROR_INVALID_VALUE = 1
    GR3_ERROR_INVALID_ATTRIBUTE = 2
    GR3_ERROR_INIT_FAILED = 3
    GR3_ERROR_OPENGL_ERR = 4
    GR3_ERROR_OUT_OF_MEM = 5
    GR3_ERROR_NOT_INITIALIZED = 6
    GR3_ERROR_CAMERA_NOT_INITIALIZED = 7
    GR3_ERROR_UNKNOWN_FILE_EXTENSION = 8
    GR3_ERROR_CANNOT_OPEN_FILE = 9
    GR3_ERROR_EXPORT = 10

class GR3_Quality(object):
    GR3_QUALITY_OPENGL_NO_SSAA  = 0
    GR3_QUALITY_OPENGL_2X_SSAA  = 2
    GR3_QUALITY_OPENGL_4X_SSAA  = 4
    GR3_QUALITY_OPENGL_8X_SSAA  = 8
    GR3_QUALITY_OPENGL_16X_SSAA = 16
    GR3_QUALITY_POVRAY_NO_SSAA  = 0+1
    GR3_QUALITY_POVRAY_2X_SSAA  = 2+1
    GR3_QUALITY_POVRAY_4X_SSAA  = 4+1
    GR3_QUALITY_POVRAY_8X_SSAA  = 8+1
    GR3_QUALITY_POVRAY_16X_SSAA = 16+1

class GR3_Drawable(object):
    GR3_DRAWABLE_OPENGL = 1
    GR3_DRAWABLE_GKS = 2

class GR3_SurfaceOption(object):
    GR3_SURFACE_DEFAULT     =  0
    GR3_SURFACE_NORMALS     =  1
    GR3_SURFACE_FLAT        =  2
    GR3_SURFACE_GRTRANSFORM =  4
    GR3_SURFACE_GRCOLOR     =  8
    GR3_SURFACE_GRZSHADED   = 16

class GR3_Exception(Exception):
    def __init__(self, error_code):
        Exception.__init__(self, geterrorstring(error_code))

def init(attrib_list=[]):
    """
    **Parameters:**

    `attrib_list` :
        a list that can specify details about context creation. The attributes which use unsigned integer values are followed by these values.

    **Raise:**

    `gr3.GR3_Error.GR3_ERROR_EXPORT`: Raises GR3_Exception

        +-----------------------------+-----------------------------------------------------------------------------------------------+
        | GR3_ERROR_NONE              | on success                                                                                    |
        +-----------------------------+-----------------------------------------------------------------------------------------------+
        | GR3_ERROR_INVALID_VALUE     | if one of the attributes' values is out of the allowed range                                  |
        +-----------------------------+-----------------------------------------------------------------------------------------------+
        | GR3_ERROR_INVALID_ATTRIBUTE | if the list contained an unkown attribute or two mutually exclusive attributes were both used |
        +-----------------------------+-----------------------------------------------------------------------------------------------+
        | GR3_ERROR_OPENGL_ERR        | if an OpenGL error occured                                                                    |
        +-----------------------------+-----------------------------------------------------------------------------------------------+
        | GR3_ERROR_INIT_FAILED       | if an error occured during initialization of the "window toolkit" (CGL, GLX, WIN...)          |
        +-----------------------------+-----------------------------------------------------------------------------------------------+

    """
    global _gr3
    _attrib_list = intarray(list(attrib_list)+[GR3_InitAttribute.GR3_IA_END_OF_LIST])
    if py_log_callback:
        lib_found = ctypes.util.find_library(os.path.join(os.path.dirname(os.path.realpath(__file__)),"libGR3.so"))
        if lib_found:
            py_log_callback("Loaded dynamic library from "+lib_found)
        else:
            py_log_callback("Loaded dynamic library unknown.")
    err = _gr3.gr3_init(_attrib_list.data)
    if err:
        raise GR3_Exception(err)

def drawimage(xmin, xmax, ymin, ymax, pixel_width, pixel_height, window):
    global _gr3
    err = _gr3.gr3_drawimage(c_float(xmin), c_float(xmax),
                       c_float(ymin), c_float(ymax),
                         c_int(pixel_width), c_int(pixel_height), c_int(window))
    if err:
        raise GR3_Exception(err)

def terminate():
    """
    This function terminates the gr3 context. After calling this function, gr3 is in the same state as when it was first loaded, except for context-independent variables, i.e. the logging callback.

    .. note:: It is safe to call this function even if the context is not initialized.

    .. seealso:: Functions :py:func:`gr.init()`
    """
    global _gr3
    _gr3.gr3_terminate()

def setquality(quality):
    """
    Set rendering quality

    **Parameters:**
        `quality`:
            The quality to set
    """
    global _gr3
    err = _gr3.gr3_setquality(quality)
    if err:
        raise GR3_Exception(err)

def getimage(width, height, use_alpha = True):
    global _gr3
    bpp = 4 if use_alpha else 3
    _bitmap = numpy.zeros((width*height*bpp), c_ubyte)
    err = _gr3.gr3_getimage(c_uint(width), c_uint(height), c_uint(use_alpha), _bitmap.ctypes.data_as(POINTER(c_ubyte)))
    if err:
        raise GR3_Exception(err)
    return _bitmap

def export(filename, width, height):
    global _gr3
    _filename = char(filename)
    err = _gr3.gr3_export(_filename, c_uint(width), c_uint(height))
    if err:
        raise GR3_Exception(err)

def geterrorstring(error_code):
    """
    **Parameters:**

        `error_code`: The error code whose representation will be returned.

    This function returns a string representation of a given error code.
    """
    for error_string in dir(GR3_Error):
        if error_string[:len('GR3_ERROR_')] == 'GR3_ERROR_':
            if GR3_Error.__dict__[error_string] == error_code:
                return error_string
    return 'GR3_ERROR_UNKNOWN'

def getrenderpathstring():
    """
    This function allows the user to find out how his commands are rendered.

    **Returns:**

    If gr3 is initialized, a string in the format: `"gr3 - " + window toolkit + " - " + framebuffer extension + " - " + OpenGL version + " - " + OpenGL renderer string`.
    For example `"gr3 - GLX - GL_ARB_framebuffer_object - 2.1 Mesa 7.10.2 - Software Rasterizer"` might be returned on a Linux system (using GLX) with an available GL_ARB_framebuffer_object implementation.
    If gr3 is not initialized `"Not initialized"` is returned.

    """
    global _gr3
    _gr3.gr3_getrenderpathstring.restype = c_char_p
    return str(_gr3.gr3_getrenderpathstring())

py_log_callback = None
log_callback = None
def setlogcallback(func):
    """
    **Parameters:**

        'func': The logging callback, a function which gets a const char pointer as its only argument and returns nothing.

    During software development it will often be helpful to get debug output from gr3. This information is not printed, but reported directly to the user by calling a logging callback. This function allows to set this callback or disable it again by calling it with `NULL`.

    """
    global log_callback, py_log_callback
    global _gr3
    LOGCALLBACK = CFUNCTYPE(None, c_char_p)
    log_callback = LOGCALLBACK(func)
    py_log_callback = func
    _gr3.gr3_setlogcallback.argtypes = [LOGCALLBACK]
    _gr3.gr3_setlogcallback.restype = None
    _gr3.gr3_setlogcallback(log_callback)


def createmesh(n, vertices, normals, colors):
    """
    This function creates a int from vertex position, normal and color data.
    Returns a mesh.

    **Parameters:**

        `vertices` : the vertex positions

        `normals` : the vertex normals

        `colors` : the vertex colors, they should be white (1,1,1) if you want to change the color for each drawn mesh

        `n` : the number of vertices in the mesh

    **Raises:**

    `gr3.GR3_Error.GR3_ERROR_EXPORT`: Raises GR3_Exception

        +----------------------+-------------------------------+
        | GR3_ERROR_NONE       | on success                    |
        +----------------------+-------------------------------+
        | GR3_ERROR_OPENGL_ERR | if an OpenGL error occured    |
        +----------------------+-------------------------------+
        | GR3_ERROR_OUT_OF_MEM | if a memory allocation failed |
        +----------------------+-------------------------------+
    """
    _mesh = c_uint(0)
    vertices = floatarray(vertices)
    normals = floatarray(normals)
    colors = floatarray(colors)
    err = _gr3.gr3_createmesh(byref(_mesh), c_uint(n), vertices.data, normals.data, colors.data)
    if err:
        raise GR3_Exception(err)
    return _mesh

def createindexedmesh(num_vertices, vertices, normals, colors, num_indices, indices):
    """
    This function creates an indexed mesh from vertex information (position,
    normal and color) and triangle information (indices).
    Returns a mesh.

    **Parameters:**

        `num_vertices` : the number of vertices in the mesh

        `vertices` : the vertex positions

        `normals` : the vertex normals

        `colors` : the vertex colors, they should be white (1,1,1) if you want to change the color for each drawn mesh

        `num_indices` : the number of indices in the mesh (three times the number of triangles)

        `indices` : the index array (vertex indices for each triangle)

    **Raises:**

    `gr3.GR3_Error.GR3_ERROR_EXPORT`: Raises GR3_Exception

        +----------------------+-------------------------------+
        | GR3_ERROR_NONE       | on success                    |
        +----------------------+-------------------------------+
        | GR3_ERROR_OPENGL_ERR | if an OpenGL error occured    |
        +----------------------+-------------------------------+
        | GR3_ERROR_OUT_OF_MEM | if a memory allocation failed |
        +----------------------+-------------------------------+
    """
    _mesh = c_uint(0)
    vertices = floatarray(vertices)
    normals = floatarray(normals)
    colors = floatarray(colors)
    indices = intarray(indices)
    err = _gr3.gr3_createindexedmesh(byref(_mesh), c_uint(num_vertices), vertices.data, normals.data, colors.data, c_uint(num_indices), indices.data)
    if err:
        raise GR3_Exception(err)
    return _mesh

def createheightmapmesh(heightmap, num_columns, num_rows):
    heightmap = floatarray(heightmap)
    return _gr3.gr3_createheightmapmesh(heightmap.data, c_int(num_columns), c_int(num_rows))

def drawheightmap(heightmap, num_columns, num_rows, positions, scales):
    heightmap = floatarray(heightmap)
    positions = floatarray(positions)
    scales = floatarray(scales)
    _gr3.gr3_drawheightmap(heightmap.data, c_int(num_columns), c_int(num_rows), positions.data, scales.data)



def drawcylindermesh(n, positions, directions, colors, radii, lengths):
    """
    This function allows drawing a cylinder without requiring a mesh.

    .. seealso::

        Function :py:func:`gr.drawmesh()`
    """
    positions = floatarray(positions)
    directions = floatarray(directions)
    colors = floatarray(colors)
    radii = floatarray(radii)
    lengths = floatarray(lengths)
    _gr3.gr3_drawcylindermesh(c_uint(n), positions.data, directions.data, colors.data, radii.data, lengths.data)

def drawconemesh(n, positions, directions, colors, radii, lengths):
    """
    This function allows drawing a cylinder without requiring a mesh.

    .. seealso::

        Function :py:func:`gr.drawmesh()`
    """
    positions = floatarray(positions)
    directions = floatarray(directions)
    colors = floatarray(colors)
    radii = floatarray(radii)
    lengths = floatarray(lengths)
    _gr3.gr3_drawconemesh(c_uint(n), positions.data, directions.data, colors.data, radii.data, lengths.data)

def drawspheremesh(n, positions, colors, radii):
    """
    This function allows drawing a sphere without requiring a mesh.

    .. seealso::

        Function :py:func:`gr.drawmesh()`
    """
    positions = floatarray(positions)
    colors = floatarray(colors)
    radii = floatarray(radii)
    _gr3.gr3_drawspheremesh(c_uint(n), positions.data, colors.data, radii.data)

def drawmesh(mesh, n, positions, directions, ups, colors, scales):
    """
    This function adds a mesh to the draw list, so it will be drawn when the user calls gr3.getpixmap_(). The given data stays owned by the user, a copy will be saved in the draw list and the mesh reference counter will be increased.

    **Parameters:**

        `mesh` : The mesh to be drawn

        `positions` : The positions where the meshes should be drawn

        `directions` : The forward directions the meshes should be facing at

        `ups` : The up directions

        `colors` : The colors the meshes should be drawn in, it will be multiplied with each vertex color

        `scales` : The scaling factors

        `n` : The number of meshes to be drawn

    .. note:

        This function does not return an error code, because of its asynchronous nature. If gr3.getpixmap_() returns a `GR3_ERROR_OPENGL_ERR`, this might be caused by this function saving unuseable data into the draw list.
    """
    positions = floatarray(positions)
    directions = floatarray(directions)
    ups = floatarray(ups)
    colors = floatarray(colors)
    scales = floatarray(scales)
    _gr3.gr3_drawmesh(mesh, c_uint(n), positions.data, directions.data, ups.data, colors.data, scales.data)

def drawcubemesh(n, positions, directions, ups, colors, scales):
    positions = floatarray(positions)
    directions = floatarray(directions)
    ups = floatarray(ups)
    colors = floatarray(colors)
    scales = floatarray(scales)
    _gr3.gr3_drawcubemesh(c_uint(n), positions.data, directions.data, ups.data, colors.data, scales.data)


def deletemesh(mesh):
    """
    This function marks a mesh for deletion and removes the user's reference from the mesh's referenc counter, so a user must not use the mesh after calling this function. If the mesh is still in use for draw calls, the mesh will not be truly deleted until gr3.clear() is called.

    **Parameters:**

        `mesh` : The mesh that should be marked for deletion

    """
    _gr3.gr3_deletemesh(mesh)

def clear():
    """
    This function clears the draw list.

    **Raises:**

    `gr3.GR3_Error.GR3_ERROR_EXPORT`: Raises GR3_Exception

        +---------------------------+-------------------------------------------------------------+
        | GR3_ERROR_NONE            | on success                                                  |
        +---------------------------+-------------------------------------------------------------+
        | GR3_ERROR_OPENGL_ERR      | if an OpenGL error occured                                  |
        +---------------------------+-------------------------------------------------------------+
        | GR3_ERROR_NOT_INITIALIZED | if the function was called without calling gr3_init() first |
        +---------------------------+-------------------------------------------------------------+


    """
    _gr3.gr3_clear()

def setbackgroundcolor(red, green, blue, alpha):
    """
    This function sets the background color.
    """
    _gr3.gr3_setbackgroundcolor(c_float(red), c_float(green), c_float(blue), c_float(alpha))

def cameralookat(camera_x, camera_y, camera_z, center_x, center_y, center_z, up_x, up_y, up_z):
    """
    This function sets the view matrix by getting the position of the camera, the position of the center of focus and the direction which should point up. This function takes effect when the next image is created. Therefore if you want to take pictures of the same data from different perspectives, you can call and  gr3.cameralookat(), gr3.getpixmap_(), gr3.cameralookat(), gr3.getpixmap_(), ... without calling gr3_clear() and gr3_drawmesh() again.

    **Parameters:**

        `camera_x` : The x-coordinate of the camera

        `camera_y` : The y-coordinate of the camera

        `camera_z` : The z-coordinate of the camera

        `center_x` : The x-coordinate of the center of focus

        `center_y` : The y-coordinate of the center of focus

        `center_z` : The z-coordinate of the center of focus

        `up_x` : The x-component of the up direction

        `up_y` : The y-component of the up direction

        `up_z` : The z-component of the up direction


    .. note::
        Source: http://www.opengl.org/sdk/docs/man/xhtml/gluLookAt.xml
            (as of 10/24/2011, licensed under SGI Free Software Licence B)

    """
    _gr3.gr3_cameralookat(c_float(camera_x), c_float(camera_y), c_float(camera_z), c_float(center_x), c_float(center_y), c_float(center_z), c_float(up_x), c_float(up_y), c_float(up_z))

def setcameraprojectionparameters(vertical_field_of_view, zNear, zFar):
    """
    This function sets the projection parameters. This function takes effect when the next image is created.

    **Parameters:**

        `vertical_field_of_view` : This parameter is the vertical field of view in degrees. It must be greater than 0 and less than 180.

        `zNear` : The distance to the near clipping plane.

        `zFar` : The distance to the far clipping plane.

    **Raises:**

    `gr3.GR3_Error.GR3_ERROR_EXPORT`: Raises GR3_Exception

        +-------------------------+-------------------------------------------------------+
        | GR3_ERROR_NONE          | on success                                            |
        +-------------------------+-------------------------------------------------------+
        | GR3_ERROR_INVALID_VALUE | if one (or more) of the arguments is out of its range |
        +-------------------------+-------------------------------------------------------+


    .. note::

        The ratio between zFar and zNear influences the precision of the depth buffer, the greater `$ \\\\frac{zFar}{zNear} $`, the more likely are errors. So you should try to keep both values as close to each other as possible while making sure everything you want to be visible, is visible.

    """
    _gr3.gr3_setcameraprojectionparameters(c_float(vertical_field_of_view), c_float(zNear), c_float(zFar))

def setlightdirection(*xyz):
    """
    This function sets the direction of light. If it is called with (0, 0, 0), the light is always pointing into the same direction as the camera.

    **Parameters:**

        `x` : The x-component of the light's direction

        `y` : The y-component of the light's direction

        `z` : The z-component of the light's direction

    """
    if len(xyz) == 3:
        x, y, z = xyz
        _gr3.gr3_setlightdirection(c_float(x), c_float(y), c_float(z))
    elif len(xyz) == 1:
        if xyz[0] is None:
            _gr3.gr3_setlightdirection(c_float(0), c_float(0), c_float(0))
        else:
            raise TypeError("if gr3_setlightdirection() is called with 1 argument, it should be None")
    else:
        raise TypeError("gr3_setlightdirection() takes exactly 1 or exactly 3 arguments (%d given)" %len(xyz))

def PyBuffer_FromMemory(address, length):
    return buffer((c_char * length).from_address(address))

def triangulate(grid, step, offset, isolevel, slices = None):
    data = grid.ctypes.data_as(POINTER(c_ushort))
    isolevel = c_ushort(isolevel)
    if slices is None:
        dim_x, dim_y, dim_z = map(c_uint, grid.shape)
        stride_x, stride_y, stride_z = (0, 0, 0)
    else:
        stride_x = grid.shape[2]*grid.shape[1]
        stride_y = grid.shape[2]
        stride_z = 1
        for i, slice in enumerate(slices):
            if slice[0] >= slice[1]:
                raise ValueError("first slice value must be less than the second value (axis=%d)" % i)
            if slice[0] < 0:
                raise ValueError("slice values must be positive (axis=%d)" % i)
            if slice[1] > grid.shape[i]:
                raise ValueError("second slice value must be at most as large as the grid dimension (axis=%d)" % i)
        dim_x, dim_y, dim_z = [c_uint(slice[1]-slice[0]) for slice in slices]
        data_offset = 2*(stride_x*slices[0][0] + stride_y*slices[1][0] + stride_z*slices[2][0])
        data_address = cast(byref(data), POINTER(c_ulong)).contents.value
        data_address += data_offset
        data = cast(POINTER(c_ulong)(c_ulong(data_address)), POINTER(POINTER(c_ushort))).contents
        offset = [offset[i] + slices[i][0]*step[i] for i in range(3)]
    stride_x = c_uint(stride_x)
    stride_y = c_uint(stride_y)
    stride_z = c_uint(stride_z)
    step_x, step_y, step_z = map(c_double, step)
    offset_x, offset_y, offset_z = map(c_double, offset)
    triangles_p = POINTER(c_float)()
    num_triangles = _gr3.gr3_triangulate(data, isolevel,
                                    dim_x, dim_y, dim_z,
                                    stride_x, stride_y, stride_z,
                                    step_x, step_y, step_z,
                                    offset_x, offset_y, offset_z,
                                    byref(triangles_p))
    if _impl != 'PyPy':
        buffer_from_memory = pythonapi.PyBuffer_FromMemory
    else:
        buffer_from_memory = PyBuffer_FromMemory
    buffer_from_memory.restype = py_object
    buffer = buffer_from_memory(triangles_p, 4*3*3*2*num_triangles)
    triangles = numpy.frombuffer(buffer, numpy.float32).copy()
    _gr3.gr3_free(triangles_p)
    triangles.shape = (num_triangles, 2, 3, 3)
    vertices = triangles[:, 0, :, :]
    normals = triangles[:, 1, :, :]
    return vertices, normals

def createisosurfacemesh(grid, step, offset, isolevel):
    """
    This function creates an isosurface from voxel data using the
    marching cubes algorithm.
    Returns a mesh.

    **Parameters:**

        `grid` : 3D numpy array containing the voxel data

        `step` : voxel sizes in each direction

        `offset` : coordinate origin in each direction

        `isolevel` : isovalue at which the surface will be created

    **Raises:**

    `gr3.GR3_Error.GR3_ERROR_EXPORT`: Raises GR3_Exception

        +----------------------+-------------------------------+
        | GR3_ERROR_NONE       | on success                    |
        +----------------------+-------------------------------+
        | GR3_ERROR_OPENGL_ERR | if an OpenGL error occured    |
        +----------------------+-------------------------------+
        | GR3_ERROR_OUT_OF_MEM | if a memory allocation failed |
        +----------------------+-------------------------------+
    """
    _mesh = c_uint(0)
    data = grid.ctypes.data_as(POINTER(c_ushort))
    isolevel = c_ushort(isolevel)
    dim_x, dim_y, dim_z = map(c_uint, grid.shape)
    stride_x, stride_y, stride_z = map(lambda x: c_uint(x / grid.itemsize), grid.strides)
    step_x, step_y, step_z = map(c_double, step)
    offset_x, offset_y, offset_z = map(c_double, offset)
    err = _gr3.gr3_createisosurfacemesh(byref(_mesh), data, isolevel,
                                        dim_x, dim_y, dim_z,
                                        stride_x, stride_y, stride_z,
                                        step_x, step_y, step_z,
                                        offset_x, offset_y, offset_z)
    if err:
        raise GR3_Exception(err)
    return _mesh

def createsurfacemesh(nx, ny, px, py, pz, option=0):
    """
    Create a mesh of a surface plot similar to gr_surface.
    Uses the current colormap. To apply changes of the colormap
    a new mesh has to be created.

    **Parameters:**

        `mesh` :    the mesh handle

        `nx` :      number of points in x-direction

        `ny` :      number of points in y-direction

        `px` :      an array containing the x-coordinates

        `py` :      an array containing the y-coordinates

        `pz` :      an array of length nx * ny containing the z-coordinates

        `option` : option for the surface mesh; the GR3_SURFACE constants can be combined with bitwise or. See the table below.

    +-------------------------+----+----------------------------------------------------------------------------------------+
    | GR3_SURFACE_DEFAULT     |  0 | default behavior                                                                       |
    +-------------------------+----+----------------------------------------------------------------------------------------+
    | GR3_SURFACE_NORMALS     |  1 | interpolate the vertex normals from the gradient                                       |
    +-------------------------+----+----------------------------------------------------------------------------------------+
    | GR3_SURFACE_FLAT        |  2 | set all z-coordinates to zero                                                          |
    +-------------------------+----+----------------------------------------------------------------------------------------+
    | GR3_SURFACE_GRTRANSFORM |  4 | use gr_inqwindow, gr_inqspace and gr_inqscale to transform the data to NDC coordinates |
    +-------------------------+----+----------------------------------------------------------------------------------------+
    | GR3_SURFACE_GRCOLOR     |  8 | color the surface according to the current gr colormap                                 |
    +-------------------------+----+----------------------------------------------------------------------------------------+
    | GR3_SURFACE_GRZSHADED   | 16 | like GR3_SURFACE_GRCOLOR, but use the z-value directly as color index                  |
    +-------------------------+----+----------------------------------------------------------------------------------------+
    """
    _mesh = c_uint(0)
    px = floatarray(px, copy=False)
    py = floatarray(py, copy=False)
    pz = floatarray(pz, copy=False)
    err = _gr3.gr3_createsurfacemesh(byref(_mesh), c_int(nx), c_int(ny),
                                     px.data,
                                     py.data,
                                     pz.data,
                                     c_int(option))

    if err:
        raise GR3_Exception(err)
    return _mesh

def drawmesh_grlike(mesh, n, positions, directions, ups, colors, scales):
    """
    Draw a mesh with the projection of gr. It uses the current
    projection parameters (rotation, tilt) of gr.
    This function alters the projection type, the projection parameters,
    the viewmatrix and the light direction. If necessary, the user has to
    save them before the call to this function and restore them after
    the call to gr3_drawimage.

    **Parameters:**

        `mesh` :       the mesh to be drawn

        `n` :          the number of meshes to be drawn

        `positions` :  the positions where the meshes should be drawn

        `directions` : the forward directions the meshes should be facing at

        `ups` :        the up directions

        `colors` :     the colors the meshes should be drawn in, it will be multiplied with each vertex color

        `scales` :     the scaling factors
    """
    positions = floatarray(positions)
    directions = floatarray(directions)
    ups = floatarray(ups)
    colors = floatarray(colors)
    scales = floatarray(scales)
    _gr3.gr3_drawmesh_grlike(mesh, c_uint(n),
                             positions.data,
                             directions.data,
                             ups.data,
                             colors.data,
                             scales.data)

def drawsurface(mesh):
    """
    Convenience function for drawing a surfacemesh

    **Parameters:**

        `mesh` : the mesh to be drawn
    """
    _gr3.gr3_drawsurface(mesh)

def surface(px, py, pz, option=0):
    """
    Create a surface plot with gr3 and draw it with gks as cellarray

    **Parameters:**

        `nx` :     number of points in x-direction

        `ny` :     number of points in y-direction

        `px` :     an array containing the x-coordinates

        `py` :     an array containing the y-coordinates

        `pz` :     an array of length nx * ny containing the z-coordinates

        `option` : see the option parameter of gr_surface. OPTION_COLORED_MESH and OPTION_Z_SHADED_MESH are supported.
    """
    if option in (gr.OPTION_Z_SHADED_MESH, gr.OPTION_COLORED_MESH):
        nx = len(px)
        ny = len(py)
        px = floatarray(px, copy=False)
        py = floatarray(py, copy=False)
        pz = floatarray(pz, copy=False)
        _gr3.gr3_surface(c_int(nx), c_int(ny),
                         px.data,
                         py.data,
                         pz.data,
                         c_int(option))
    else:
        gr.surface(px, py, pz, option)
    return

_gr3.gr3_init.argtypes = [POINTER(c_int)]
_gr3.gr3_terminate.argtypes = []
_gr3.gr3_getrenderpathstring.argtypes = []
_gr3.gr3_geterrorstring.argtypes = [c_int]
_gr3.gr3_setlogcallback.argtypes = [CFUNCTYPE(None, c_char_p)]
_gr3.gr3_clear.argtypes = []
_gr3.gr3_setquality.argtypes = [c_int]
_gr3.gr3_getimage.argtypes = [c_int, c_int, c_int, POINTER(c_ubyte)]
_gr3.gr3_export.argtypes = [POINTER(c_char), c_uint, c_uint]
_gr3.gr3_drawimage.argtypes = [c_float, c_float, c_float, c_float,
                               c_int, c_int, c_int]
_gr3.gr3_createmesh.argtypes = [POINTER(c_uint), c_uint, POINTER(c_float),
                                POINTER(c_float), POINTER(c_float)]
_gr3.gr3_createindexedmesh.restype = c_int
_gr3.gr3_createindexedmesh.argtypes = [POINTER(c_uint), c_uint,
                                       POINTER(c_float), POINTER(c_float),
                                       POINTER(c_float), c_uint, POINTER(c_int)]
_gr3.gr3_createheightmapmesh.argtypes = [POINTER(c_float), c_int, c_int]
_gr3.gr3_drawheightmap.argtypes = [POINTER(c_float), c_int, c_int,
                                   POINTER(c_float), POINTER(c_float)]
_gr3.gr3_drawmesh.argtypes = [c_uint, c_uint, POINTER(c_float),
                              POINTER(c_float), POINTER(c_float),
                              POINTER(c_float), POINTER(c_float)]
_gr3.gr3_deletemesh.argtypes = [c_int]

_gr3.gr3_cameralookat.argtypes = [c_float, c_float, c_float,
                                  c_float, c_float, c_float,
                                  c_float, c_float, c_float]
_gr3.gr3_setcameraprojectionparameters.argtypes = [c_float, c_float,
                                                   c_float]
_gr3.gr3_setlightdirection.argtypes = [c_float, c_float, c_float]
_gr3.gr3_setbackgroundcolor.argtypes = [c_float, c_float, c_float,
                                        c_float]
_gr3.gr3_drawconemesh.argtypes = [c_uint, POINTER(c_float),
                                  POINTER(c_float), POINTER(c_float),
                                  POINTER(c_float), POINTER(c_float)]
_gr3.gr3_drawcylindermesh.argtypes = [c_uint, POINTER(c_float),
                                      POINTER(c_float), POINTER(c_float),
                                      POINTER(c_float), POINTER(c_float)]
_gr3.gr3_drawspheremesh.argtypes = [c_uint, POINTER(c_float),
                                    POINTER(c_float), POINTER(c_float)]
_gr3.gr3_drawcubemesh.argtypes = [c_uint, POINTER(c_float),
                                  POINTER(c_float), POINTER(c_float),
                                  POINTER(c_float), POINTER(c_float)]
_gr3.gr3_triangulate.restype = c_uint
_gr3.gr3_triangulate.argtypes = [POINTER(c_ushort), c_ushort, c_uint,
                                 c_uint, c_uint, c_uint, c_uint, c_uint,
                                 c_double, c_double, c_double, c_double,
                                 c_double, c_double,
                                 POINTER(POINTER(c_float))]

_gr3.gr3_createisosurfacemesh.restype = c_int
_gr3.gr3_createisosurfacemesh.argtypes = [POINTER(c_uint),
                                          POINTER(c_ushort), c_ushort,
                                          c_uint, c_uint, c_uint,
                                          c_uint, c_uint, c_uint,
                                          c_double, c_double, c_double,
                                          c_double, c_double, c_double]

_gr3.gr3_createsurfacemesh.argtypes = [POINTER(c_uint), c_int, c_int,
                                       POINTER(c_float), POINTER(c_float),
                                       POINTER(c_float), c_int]
_gr3.gr3_createsurfacemesh.restype = c_int
_gr3.gr3_drawmesh_grlike.argtypes = [c_uint, c_uint, POINTER(c_float),
                                     POINTER(c_float), POINTER(c_float),
                                     POINTER(c_float), POINTER(c_float)]
_gr3.gr3_drawmesh_grlike.restype = None
_gr3.gr3_drawsurface.argtypes = [c_uint]
_gr3.gr3_drawsurface.restype = None
_gr3.gr3_surface.argtype = [c_int, c_int, POINTER(c_float),
                            POINTER(c_float), POINTER(c_float), c_int]
_gr3.gr3_surface.restype =  None
